/************************************************/
/*      A.Pique                                 */
/************************************************/
/*      A.Pique                                 */
/************************************************/
/*     mer fv 13 12:43:41 CET 2002             */
/************************************************/
/*     mer fv 13 12:43:41 CET 2002             */
/************************************************/
/*      Numero de version et commentaire        */
/*      0.1     Cration                        */
/************************************************/
/*      expclasse.h                            */
/************************************************/

#ifndef EXPCLASSE_H
#define EXPCLASSE_H

#include <cctype>
#include <string>
#include <vector>
#include <iostream>
#include <fstream>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include <errno.h>


#include "expobjetfichier.h"


#ifndef EXPOBJETFICHIER_H
class ExpObjetFichier;
#endif
#ifndef EXPRELATION_H
class ExpRelation ;
#endif
#ifndef EXPMETHODE_H
class ExpMethode ;
#endif
#ifndef EXPATTRIBUT_H
class ExpAttribut ;
#endif
#ifndef EXPENTETEARG_H
class ExpEnteteArg ;
#endif

#ifndef EXPINCLUDE_H
class ExpInclude ;
#endif

#ifndef NOYVISITEURNOEUD_H
class VisiteurNoeud;
#endif
#ifndef NOYCLASSE_H
class Classe;
#endif
#ifndef NOYRELATION_H
class Relation;
#endif
#ifndef NOYUTILISATION_H
class Utilisation;
#endif
#ifndef NOYAGREGATION_H
class Agregation;
#endif
#ifndef NOYHERITAGE_H
class Heritage;
#endif
#ifndef NOYASSOCIATION_H
class Association;
#endif

/**
 * Classe reprsentant une classe
 * @author A.Pique
 */
class ExpClasse: public ExpObjetFichier {

 private:
  //! Commentaire li  la classe
  string _commentaire;
  //! Cartouche de la classe
  string _cartouche;
  //! Nom de la classe
  string _nom;
  //! Visibilit de la classe
  string _visibilite;
  //! Chemin de la classe
  string _relPath;
  //! Boolen determinant si la classe est une interface(true) ou une classe(false)
  bool   _interface;
  //! Pointeur vers la classe du noyau correspondante
  Classe * _classe;
  //! Pointeur sur l'objet contenant cette classe
  ExpObjetFichier * _pere;
  //! Pointeur sur l'objet de base de l'exportation
  ExpObjetFichier * _pepe;
  //! Chane contenant les noms de toutes les paquetages pres du paquetage en cours spares par des "::"
  string _paquetages;
  //! Vecteur contenant les attributs appartenant  cette classe
  std::vector<ExpAttribut> _listeAttribut;
  //! Vecteur contenant les noms des classes dont cette classe hrite
  std::vector<ExpEnteteArg> _listeEnteteArg;
  //! Vecteur contenant les mthodes appartenant  cette classe
  std::vector<ExpMethode> _listeMethode;
  //! Vecteur contenant les includes ncessaires  cette classe
  std::vector<ExpInclude> _listeInclude;
 protected:

 public:

  /**
   * Constructeur
   * \param ExpObjetFichier * objetFichier : objet contenant cette classe
   */
  ExpClasse ( ExpObjetFichier * objetFichier );

  /**
   * Destructeur
   */
  ~ExpClasse ( );

  /**
   * Fonction initialisant la classe  partir d'une classe du noyau
   * \param Classe * classe : classe venant du noyau
   */
  void setClasse ( Classe * classe );

  /**
   * Fonction dmarrant la gnration de la classe au format C++
   * \return 0 si bon, -1 en cas d'chec
   */
  int getCpp (  );

  /**
   * Fonction dmarrant la gnration de la classe au format Java
   * \return 0 si bon, -1 en cas d'chec
   */
  int getJava (  );

  /**
   * Modificateur de l'attribut _commentaire
   * \param string commentaire : nouvelle valeur de commentaire
   */   
  void setCommentaire ( string commentaire );

  /** Accesseur de _commentaire
   * \return Le commentaire
   */  
  string getCommentaire (  );

  /**
   * Modificateur de l'attribut _cartouche
   * \param string fichier : nom de la classe
   */   
  void setCartouche ( string fichier );

  /** Accesseur de _cartouche
   * \return Le cartouche
   */  
  string getCartouche (  );

  /**
   * Modificateur de l'attribut _nom
   * \param string nom : nouvelle valeur de nom
   */  
  void setNom ( string nom );

  /** Accesseur de _nom
   * \return Le nom
   */  
  string getNom (  );

  /** Accesseur de _relPath
   * \return le chemin relatif de la classe
   */ 
  string getRelPath ();

  /** Fonction permettant de rcuprer le chemin relatif d'une classe
   * \param string path : chane de caractre
   * \return string : chemin relatif de la classe
   */  
  string getRelPath (string path);

  /** Fonction permettant de rcuprer le chemin relatif d'une classe
   * par rapport au chemin pass en paramtre
   * \param string path : chemin courant
   * \return string : chemin relatif de la classe
   */  
  string getRelatifPath (string pathCourant);

  /**
   * Modificateur de l'attribut _visibilite
   * \param string visibilite : nouvelle valeur de visibilite
   */  
  void setVisibilite ( string visibilite );

  /** Accesseur de _visibilite
   * \return La visibilite de la methode
   */ 
  string getVisibilite (  );

  /**
   * Modificateur de l'attribut _interface
   * \param string interface : nouvelle valeur d' _interface
   */  
  void setInterface(bool interface);

 /** Accesseur de _interface
   * \return L'interface de la methode
   */ 
  bool getInterface();

  /** Accesseur de _classe
   * \return la classe du noyau associe  cette classe
   */ 
  Classe *getClasse();

  /**
   * Fonction permettant l'ajout d'une mthode constructeur  la classe
   */
  void ajouterConstructeur();

  /**
   * Fonction permettant l'ajout d'une mthode destructeur  la classe
   */
  void ajouterDestructeur();

  /**
   * Fonction traitant les mthodes associes  cette classe contenues
   * dans un vecteur de mthode du noyau
   * \param vector<Methode*> vctMethode : vecteur de mthodes du noyau
   */
  void traiterLesMethodes (vector<Methode*> vctMethode);

  /**
   * Fonction traitant les attributs associs  cette classe contenus
   * dans un vecteur d'attributs du noyau
   * \param vector<Attribut*> vctAttribut : vecteur d'attributs du noyau
   */
  void traiterLesAttributs (vector<Attribut*> vctAttribut);

  /**
   * Mthode utilise pour visiter les agrgations du noyau et
   * effectuer leur traitement
   * \param Agregation *a : agrgation du noyau
   */
  void traiterAgregation ( Agregation *a );

  /**
   * Mthode utilise pour visiter les utilisations du noyau et
   * effectuer leur traitement
   * \param Utilisation *u : utilisation du noyau
   */
  void traiterUtilisation ( Utilisation *u );

  /**
   * Mthode utilise pour visiter les hritages du noyau et
   * effectuer leur traitement
   * \param Heritage *h : hritage du noyau
   */
  void traiterHeritage ( Heritage *h );

  /**
   * Mthode utilise pour visiter les associations du noyau et
   * effectuer leur traitement
   * \param Association *a : association du noyau
   */
  void traiterAssociation ( Association *a );

  /**
   * Fonction permettant de crer des chanes de "num" espaces
   * \param int num : nombre d'espaces
   * \return une chane de "num" blancs
   */
  string indent(int num);

  void ajouterInclude (ExpClasse * classeSource, ExpClasse * classeDest, bool heritage,int typeInclude);

  void ajouterEnteteArg (bool type, ExpClasse * classe);

  /**
   * Fonction permettant de convertir une chane en majuscules
   * \param string chaine : chaine  convertir
   * \return la chane convertie
   */
  string Majuscules (string chaine);

  /**
   * Fonction permettant de convertir une chane en minuscules
   * \param string chaine : chaine  convertir
   * \return la chane convertie
   */
  string Minuscules (string chaine);

  /** Fonction renvoyant la chane utilise pour le #ifndef des fichiers d'inclusion en C++
   * \return la chane en question
   */
  string getIfndef ();

  /** Accesseur de l'attribut _paquetages
   * \return la liste des paquetages pres spars par des "::"
   */
  string getPaquetage();

   /**Fonction retournant un pointeur sur la classe mre de l'export
    * \return ExpObjetFichier * : pointeur sur la classe mre
    */
  ExpObjetFichier * getPepe();

   //! Fonction initialisant l'attribut _pepe
  void setPepe();

  /** Fonction renvoyant la chane contenue dans _paquetages
   * en remplaant les dlimiteurs "::" par le nouveau dlimiteur
   * \param string delimiteur : nouveau dlimiteur
   * \return chane convertie
   */
  string getPaquetagePath(string delimiteur);

  /** Fonction inutile, dclare uniquement car elle n'est pas code
   *  Dans l'interface ExpObjetFichier
   */
  ExpPaquetage * getPaquetage(string paquetage);

  /** Fonction inutile, dclare uniquement car elle n'est pas code
   *  Dans l'interface ExpObjetFichier
   */
  ExpClasse * getClasse(string classe);//inutile

  /** Fonction permettant l'ajout d'un attribut  la classe
   * \param ExpAttribut attribut : attribut  ajouter
   */
  void ajouterAttribut(ExpAttribut attribut);

  /**
   * Fonction permettant de rcuprer la liste des "extends" et "implements" d'une classe en Java
   * \return chane de caractre  include dans la dclaration de la classe Java
   */
  string getJavaEnteteArg();

};

#endif
