/************************************************/
/*      A.Pique                                 */
/************************************************/
/*      A.Pique                                 */
/************************************************/
/*     mer fv 13 12:43:43 CET 2002             */
/************************************************/
/*     mer fv 13 12:43:43 CET 2002             */
/************************************************/
/*      Numero de version et commentaire        */
/*      0.1     Cration                        */
/************************************************/
/*      expexport.h                            */
/************************************************/

#ifndef EXPEXPORT_H
#define EXPEXPORT_H

#include <cctype>
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include <errno.h>

#include "expobjetfichier.h"

#ifndef EXPPAQUETAGE_H
class ExpPaquetage;
#endif
#ifndef EXPCLASSE_H
class ExpClasse;
#endif
#ifndef NOYVISITEURNOEUD_H
class NoyVisiteurNoeud;
#endif
#ifndef NOYPAQUETAGE_H
class NoyPaquetage;
#endif
#ifndef NOYCLASSE_H
class NoyClasse;
#endif
#ifndef NOYDIAGRAMMESTATIQUE_H
class NoyDiagrammeStatique;
#endif
/**
 * Classe principale de l'exportation, elle regroupe 
 * toutes les classes et paquetages du projet
 * @author A.Pique
 */
class ExpExport: public ExpObjetFichier {

 private:
  //! Vecteur contenant les classes n'appartenant  aucun paquetage
  vector<ExpClasse> _listeClasse;
  //! Vecteur contenant les paquetages n'appartenant  aucun autre paquetage
  vector<ExpPaquetage> _listePaquetage;
  //! Cartouche balis commun  tous les fichiers
  string _cartouche;
  //! Pointeur vers le diagramme statique du projet
  DiagrammeStatique * _diagramme;
  
  //! Rpertoire o seront crs les fichiers 
  string _path;

  protected:

  public:
  /** Constructeur
   * \param DiagrammeStatique * diagramme : diagramme statique du projet
   * \param string path : Rpertoire o seront crs les fichiers 
   */
  ExpExport ( DiagrammeStatique * diagramme,  string path );
  //! Destructeur 
  ~ExpExport ( );
  /**
   * Fonction dmarrant la conversion du diagramme statique du noyau
   * dans une structure plus adapte  l'exportation en fichiers C++ 
   * ou Java 
   */
  void remplir();
  /**
   * Modificateur de l'attribut _diagramme
   * \param DiagrammeStatique * diagramme : le nouveau diagramme statique
   */   
  void setDiagramme ( DiagrammeStatique * diagramme );
  /** Accesseur de _diagramme
   * \return le diagramme statique du noyau
   */
  DiagrammeStatique * getDiagramme (  );

  /** Fonction dmarrant l'exportation en Cpp,
   *  la fonction remplir doit avoir t appele prcedemment.
   */
  void getCpp ();
  /** Fonction dmarrant l'exportation en Java,
   *  la fonction remplir doit avoir t appele prcedemment.
   */
  void getJava ();

  /**
   * Mthode utilise pour visiter les paquetages du noyau et
   * effectuer leur traitement
   * \param Paquetage * pa : paquetage issu du noyau
   */
  void traiterPaquetage ( Paquetage * pa );
  /**
   * Mthode utilise pour visiter les classes du noyau et
   * effectuer leur traitement
   * \param Classe * cl : classe issue du noyau
   */
  void traiterClasse ( Classe * cl );

  /** Accesseur de _cartouche
   * \return Le cartouche
   */  
  string getCartouche (  );

  /** Fonction inutile, dclare uniquement car elle n'est pas code
   *  Dans l'interface ExpObjetFichier
   */
  void setCartouche ( string nomFichier ); //inutile

  /**
   * Modificateur de l'attribut _cartouche
   * \param string nomProjet : Nom du projet
   * \param string nomCreateur : Nom du crateur du projet
   */   
  void setCartouche ( string nomProjet, string nomCreateur ) ;
  
  /** Fonction rcuprant le chemin d'accs au projet
   * \return le chemin d'accs au projet
   */
  string getPath (  );
  
  /** Fonction permettant de modifier le chemin d'accs du projet
   * \param string path : chane contenant le chemin
   */
  void setPath ( string path ) ;

  /** Fonction permettant de rcuprer le chemin relatif d'une classe
   * \param string path : chane de caractre
   * \return string : chemin relatif de la classe
   */  
  string getRelPath (string path);

  /**Fonction permettant d'obtenir le nom complet du paquetage,
   * ici, puisqu'il s'agit de la racine, ce nom est une chaine vide ("")
   * \return string : le nom complet du paquetage
   */  
  string getPaquetage();

  /**Fonction retournant un pointeur sur la classe mre de l'export
   * \return ExpObjetFichier * : pointeur sur la classe mre
   */
  ExpObjetFichier * getPepe();

  /** Fonction inutile, dclare uniquement car elle n'est pas code
   *  Dans l'interface ExpObjetFichier
   */
  string getNom(); //inutile

  /**Fonction retournant un pointeur sur un ExpPaquetage s'il appartient  un vecteur de ExpPaquetage de l'objet
   * \param string paquetage : nom du paquetage
   * \return ExpPaquetage * : pointeur sur le paquetage ou null s'il n'existe pas
   */
  ExpPaquetage * getPaquetage(string paquetage);

  /**Fonction retournant un pointeur sur un ExpClasse s'il appartient  un vecteur de ExpClasse de l'objet
   * \param string classe : nom de la classe
   * \return ExpClasse * : pointeur sur la classe ou null si elle n'existe pas
   */
  ExpClasse * getClasse(string classe);
  
};

#endif
