/************************************************/
/*      A.Pique                                 */
/************************************************/
/*      A.Pique                                 */
/************************************************/
/*     mer fv 13 12:43:44 CET 2002             */
/************************************************/
/*     mer fv 13 12:43:44 CET 2002             */
/************************************************/
/*      Numero de version et commentaire        */
/*      0.1     Cration                        */
/************************************************/
/*      exppaquetage.h                          */
/************************************************/

#ifndef EXPPAQUETAGE_H
#define EXPPAQUETAGE_H

#include <cctype>
#include <string>
#include <iostream>
#include <fstream>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include <errno.h>
#include <vector>

#include "expobjetfichier.h"

#ifndef EXPCLASSE_H
class ExpClasse;
#endif
#ifndef EXPPAQUETAGE_H
class ExpPaquetage;
#endif

#ifndef NOYPAQUETAGE_H
class Paquetage;
#endif
#ifndef NOYVISITEURNOEUD_H
class VisiteurNoeud;
#endif

/**
 * Classe reprsentant un paquetage
 * @author A.Pique
 */
class ExpPaquetage: public ExpObjetFichier {

 private:
  //! Commentaire li au paquetage
  string _commentaire;
  //! Cartouche du paquetage
  string _cartouche;
  //! Nom du paquetage
  string _nom;
  //! Vecteur contenant les classes appartenant  ce paquetage
  vector<ExpClasse> _listeClasse;
  //! Vecteur contenant les paquetages appartenant  ce paquetage
  vector<ExpPaquetage> _listePaquetage;
  //! Vecteur contenant les fichiers d'inclusion des classes et paquetages du paquetage
  vector<string> _listeInclude;
  //! Pointeur vers le paquetage du noyau correspondant
  Paquetage * _paquetage;
  //! Pointeur sur l'objet contenant ce paquetage
  ExpObjetFichier * _pere;
  //! Pointeur sur l'objet de base de l'exportation
  ExpObjetFichier * _pepe;
  //! Chane contenant les noms de toutes les paquetages pres du paquetage en cours spares par des "::"
  string _paquetages;
 protected:

 public:
  /** Constructeur
   * \param ExpObjetFichier * pere : pointeur sur l'objet contenant le paquetage en cours
   */
  ExpPaquetage (ExpObjetFichier * pere);
  //! Destructeur
  ~ExpPaquetage();
  /**
   * Modificateur de l'attribut _paquetage
   * \param Paquetage * paquetage : le nouveau paquetage
   */   
  void setPaquetage ( Paquetage * paquetage );
  /** Accesseur de _paquetage
   * \return le paquetage du noyau
   */
  string getPaquetage();

  /**Fonction retournant un pointeur sur un ExpPaquetage s'il appartient  un vecteur de ExpPaquetage de l'objet
   * \param string paquetage : nom du paquetage
   * \return ExpPaquetage * : pointeur sur le paquetage ou null s'il n'existe pas
   */
  ExpPaquetage * getPaquetage(string paquetage);
  
  /**Fonction retournant un pointeur sur un ExpClasse s'il appartient  un vecteur de ExpClasse de l'objet
   * \param string classe : nom de la classe
   * \return ExpClasse * : pointeur sur la classe ou null si elle n'existe pas
   */
  ExpClasse * getClasse(string classe);

  /** Fonction dmarrant l'exportation en Cpp du paquetage
   */
  void getCpp (  );

  /** Fonction dmarrant l'exportation en Java du paquetage
   */
  void getJava (  );

  /**
   * Modificateur de l'attribut _cartouche
   * \param string fichier : nom du paquetage
   */   
  void setCartouche ( string fichier );

  /** Accesseur de _cartouche
   * \return Le cartouche
   */  
   string getCartouche (  );

    /**
   * Modificateur de l'attribut _commentaire
   * \param string commentaire : nouvelle valeur de commentaire
   */   
   void setCommentaire ( string commentaire );

  /** Accesseur de _commentaire
   * \return Le commentaire
   */  
   string getCommentaire (  );

  /**
   * Modificateur de l'attribut _nom
   * \param string nom : nouvelle valeur de nom
   */  
   void setNom ( string nom );

   /** Accesseur de _nom
    * \return Le nom
    */  
   string getNom (  );

  /** Fonction permettant de rcuprer le chemin relatif d'une classe
   * \param string path : chane de caractre
   * \return string : chemin relatif de la classe
   */  
   string getRelPath (string path); 

  /**
   * Mthode utilise pour visiter les paquetages du noyau et
   * effectuer leur traitement
   * \param Paquetage * pa : paquetage issu du noyau
   */
   void traiterPaquetage ( Paquetage * pa );

  /**
   * Mthode utilise pour visiter les classes du noyau et
   * effectuer leur traitement
   * \param Classe * cl : classe issue du noyau
   */
   void traiterClasse ( Classe * cl );

   /**
    * Fonction permettant de crer des chanes de "num" espaces
    * \param int num : nombre d'espaces
    * \return une chane de "num" blancs
    */
   string indent(int num);

   /**
   * Fonction permettant de convertir une chane en majuscules
   * \param string chaine : chaine  convertir
   * \return la chane convertie
   */
   string Majuscules (string chaine);

   /**
   * Fonction permettant de convertir une chane en minuscules
   * \param string chaine : chaine  convertir
   * \return la chane convertie
   */
   string Minuscules (string chaine);

   /** Fonction renvoyant la chane utilise pour le #ifndef des fichiers d'inclusion en C++
    * \return la chane en question
    */
   string getIfndef ();

   /** Fonction permettant de rajouter un fichier d'inclusion  un paquetage
    * \param string nom : nom du fichier  inclure
    */
   void ajouterInclude (string nom);

   /**Fonction retournant un pointeur sur la classe mre de l'export
    * \return ExpObjetFichier * : pointeur sur la classe mre
    */
   ExpObjetFichier * getPepe();
   
   //! Fonction initialisant l'attribut _pepe
   void setPepe();
};

#endif
