#ifndef ARBRE_H
#define ARBRE_H

#include <qwidget.h>
#include <qlistview.h>
#include <iostream.h>

#include <noynoyau.h>
#include <noyobservateur.h>
#include <ihmvisiteurarbre.h>
#include <ihmmonlistviewitem.h>

/** Cette classe represente l'objet dans lequel va se dessiner l'arbre.
 * Elle ecoute le noyau et s'occupe des evenements auxquels peut tre sujet
 * l'arbre.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class Arbre : public QListView, public Observateur
{
	Q_OBJECT

	private:
		Noyau *noyau;
		Projet *p;
		VisiteurArbre *v;
		MonListViewItem *itemCourant;

	public:

		/** Constructeur
		 * \param Noyau *n : Represente le noyau associ au projet.
		 * \param QWidget *parent : Represente le widget parent de cet arbre (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  notre arbre (Par defaut 0).
		 */
		Arbre(Noyau *n, QWidget *parent=0, const char *name=0);

		//! Destructeur
		virtual ~Arbre() {}


		/** Change la valeur de la variable prive itemCourant.
		 * \param MonListViewItem *item : Nouvelle valeur de itemCourant.
		 */
		void setItemCourant(MonListViewItem *item);
		
		/** Permet de recuperer la valeur actuelle de la variable prive itemCourant.
		 * \return Une reference sur la valeur de itemCourant.
		 */
		MonListViewItem *getItemCourant();


		MonListViewItem *itemASelectionner;


		/** Cette methode averti l'arbre qu'il y a eu un changement dans le noyau.
		 * Elle provoque le raffraichissement de l'arbre.
		 */
		void avertir();

		/** Permet de recuperer le noyau qu'coute cet arbre.
		 * \return Une reference sur le noyau qu'coute cet arbre.
		 */
		Noyau *getNoyau();

	public slots:
		/** Ce slot est appel lorsqu'un clique droit survient dans l'arbre.
		 * \param QListViewItem * : Represente le QListViewItem de l'arbre sur lequel le clique est survenu.
		 */
		void cliqueDroit(QListViewItem *);

		/** Ce slot est appel lorsque l'item en tat selectionn change.
		 * \param QListViewItem * : Represente le QListViewItem de l'arbre nouvellement selectionn.
		 */
		void selectionChangee(QListViewItem *);

		//! Ce slot est appel pour envoyer un paquetage existant vers une vue passe en parametre.
		void paquetageVersVue();

		//! Ce slot est appel pour envoyer une classe existant vers une vue passe en parametre.
		void classeVersVue();

		//! Ce slot est appel pour envoyer une relation existant vers une vue passe en parametre.
		void relationVersVue();

		//! Ce slot est appel pour editer le code d'une methode
		void editionCode();

};

#endif

