#include <qfiledialog.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qtextbrowser.h>
#include <qmessagebox.h>

#include <ihmbarredemenu.h>
#include <ihmfenetreprincipale.h>
#include <ihmfenetrecreerprojet.h>
#include <ihmfenetrepreference.h>
#include <ihmfenetrevue.h>
#include <ihmfenetrepaquetage.h>
#include <ihmfenetreclasse.h>
#include <ihmfenetrecommentaire.h>
#include <ihmfenetremethode.h>
#include <ihmfenetreattribut.h>
#include <ihmfenetrerelation.h>
#include <ihmfenetreparseurjava.h>
#include <ihmfenetreparseurcpp.h>
#include <ihmfenetreexport.h>
#include <ihmfenetreaide.h>
#include <ihmvisiteurajouter.h>
#include <noyprojet.h>
#include <noycommun.h>
#include <noynoeud.h>

/**********************************************************
 *Ceci est le constructeur de la classe : cration des entres
 *de menu connectes  un SLOT (voir doc QT), des sous-menus
 *et construction du tout. De plus, la barre de menu s'abonne
 *au noyau afin d'etre avertie des changements
**********************************************************/
BarreDeMenu::BarreDeMenu(Noyau * n, QWidget *parent, const char *name)
	: QMenuBar(parent , name), Observateur()
{
	noeudEnMemoire = NULL;
	couperEnMemoire = false;

	visiteurSupprimer = new VisiteurSupprimer ( (FenetrePrincipale *)topLevelWidget() );
	noyau = n;
	noyau->abonner(this);

	projet = new QPopupMenu(this);
	edition = new QPopupMenu(this);
	outils = new QPopupMenu(this);
	ajouter = new QPopupMenu(outils);
	exporter = new QPopupMenu(outils);
	parser = new QPopupMenu(outils);
	QPopupMenu *aide = new QPopupMenu(this);

	// Projet
	CHECK_PTR(projet);
	ouvrirID = projet->insertItem("&Ouvrir", this, SLOT(ouvrir()), CTRL+Key_O);
	nouveauID = projet->insertItem("&Nouveau", this, SLOT(nouveau()), CTRL+Key_N);
	enregistrerID = projet-> insertItem("Enregi&strer", this, SLOT(enregistrer()), CTRL+Key_S);
	cloreID = projet-> insertItem("Fermer", this, SLOT(fermer()), CTRL+Key_W);	
		projet->insertSeparator();
	informationsID = projet-> insertItem("&Informations", this, SLOT(informations()), CTRL+Key_I);
		projet->insertSeparator();
	quitterID = projet->insertItem("&Quitter", this, SLOT(quitter()), CTRL+Key_Q);

	// Edition
	CHECK_PTR(edition);
	couperID = edition->insertItem("Couper", this, SLOT(couper()), CTRL+Key_X);
	copierID = edition->insertItem("&Copier", this, SLOT(copier()), CTRL+Key_C);
	collerID = edition->insertItem("Coller", this, SLOT(coller()), CTRL+Key_V);
	supprimerID = edition->insertItem("Supprimer", this, SLOT(supprimer()), CTRL+Key_D);
		edition->insertSeparator();
	preferenceID = edition->insertItem("&Preferences...", this, SLOT(preferences()), CTRL+Key_P);

	// Outils/Ajouter
	CHECK_PTR(ajouter);
	ajoutVueID = ajouter->insertItem("1-vue", this, SLOT(vue()), CTRL+Key_1);
	ajouter->insertSeparator();
	ajoutPaquetageID = ajouter->insertItem("2-paquetage", this, SLOT(paquetage()), CTRL+Key_2);
	ajoutCommentaireID = ajouter->insertItem("3-commentaire", this, SLOT(commentaire()), CTRL+Key_3);
	ajoutClasseID = ajouter->insertItem("4-classe", this, SLOT(classe()), CTRL+Key_4);
	ajoutMethodeID = ajouter->insertItem("5-methode", this, SLOT(methode()), CTRL+Key_5);
	ajoutAttributID = ajouter->insertItem("6-attribut", this, SLOT(attribut()), CTRL+Key_6);
	ajoutRelationID = ajouter->insertItem("7-relation", this, SLOT(relation()), CTRL+Key_7);

	// Outils/exporter
	CHECK_PTR(exporter);
	exportCppID = exporter->insertItem("C++", this, SLOT(exportCpp())); 
	exportJavaID = exporter->insertItem("Java", this, SLOT(exportJava()));
	
	// Outils/parser
	CHECK_PTR(parser);
	importCppID = parser->insertItem("C++", this, SLOT(importCpp()));
	importJavaID = parser->insertItem("Java", this, SLOT(importJava()));
	
	//Outils
	CHECK_PTR(outils);
	outils->insertItem("Ajouter", ajouter);
	outils->insertSeparator();
	outils->insertItem("Exporter", exporter);
	outils->insertItem("Parser", parser);

	// Aide
	CHECK_PTR(aide);
	aide->insertItem("&Aide", this, SLOT(aide()), CTRL+Key_H);
	aide->insertSeparator();
	aide->insertItem("&A propos...", this, SLOT(aPropos()), CTRL+Key_A);

	// Creation de la barre de menu et association des popup
	CHECK_PTR(this);
	this->insertItem("Projet", projet);
	this->insertItem("Edition", edition);
	this->insertItem("Outils", outils);
	this->insertSeparator();
	this->insertItem("Aide", aide);
	this->setSeparator( QMenuBar::InWindowsStyle );
}

/**********************************************************
 *cette fonction met  jour les entres de menu. Cette classe
 *est observatrice du noyau, c'est  dire qu' chaque changement
 *dans le noyau, celui-ci la prvient afin de se mettre  jour
**********************************************************/
void BarreDeMenu::avertir() {
	Projet * p = noyau->getProjet();
	if ( p == NULL ) {
		/* dsactivation des entres de menu */
		projet->setItemEnabled(ouvrirID,TRUE);
		projet->setItemEnabled(nouveauID,TRUE);
		projet->setItemEnabled(informationsID,FALSE);
		projet->setItemEnabled(enregistrerID,FALSE);
		projet->setItemEnabled(cloreID,FALSE);
		
		edition->setItemEnabled(couperID,FALSE);
		edition->setItemEnabled(copierID,FALSE);
		edition->setItemEnabled(collerID,FALSE);
		edition->setItemEnabled(supprimerID,FALSE);
		edition->setItemEnabled(preferenceID,FALSE);	

		ajouter->setItemEnabled(ajoutVueID,FALSE);
		ajouter->setItemEnabled(ajoutPaquetageID,FALSE);
		ajouter->setItemEnabled(ajoutCommentaireID,FALSE);
		ajouter->setItemEnabled(ajoutClasseID,FALSE);
		ajouter->setItemEnabled(ajoutMethodeID,FALSE);
		ajouter->setItemEnabled(ajoutAttributID,FALSE);
		ajouter->setItemEnabled(ajoutRelationID,FALSE);

		exporter->setItemEnabled(exportCppID,FALSE);
		exporter->setItemEnabled(exportJavaID,FALSE);
		parser->setItemEnabled(importCppID,FALSE);
		parser->setItemEnabled(importJavaID,FALSE);
	} else {
		/* projet existe donc */

		/* 1 : on peut acceder au informations et le sauvegarder */
		projet->setItemEnabled(informationsID,TRUE);
		projet->setItemEnabled(enregistrerID,TRUE);
		projet->setItemEnabled(cloreID,TRUE);
		projet->setItemEnabled(ouvrirID,FALSE);
		projet->setItemEnabled(nouveauID,FALSE);
		edition->setItemEnabled(preferenceID,TRUE);

		/* 2 : on peut ajouter des paquetage, des classes et des commentaires */
		ajouter->setItemEnabled(ajoutVueID,TRUE);
		ajouter->setItemEnabled(ajoutPaquetageID,TRUE);
		ajouter->setItemEnabled(ajoutCommentaireID,TRUE);
		ajouter->setItemEnabled(ajoutClasseID,TRUE);
		ajouter->setItemEnabled(ajoutMethodeID,TRUE);
		ajouter->setItemEnabled(ajoutAttributID,TRUE);
		ajouter->setItemEnabled(ajoutRelationID,TRUE);

		/* 3 : si un objet est slectionn */
		if ( ((FenetrePrincipale *)topLevelWidget())->getNoeudCourant() != NULL ) {
				edition->setItemEnabled(couperID,TRUE);
				edition->setItemEnabled(copierID,TRUE);
				edition->setItemEnabled(collerID,TRUE);
				edition->setItemEnabled(supprimerID,TRUE);
		}

		/* 4 : on peut communiquer avec le parseur et l'exportateur */
		
		if (p->getLangage() == true){
			exporter->setItemEnabled(exportCppID,TRUE);
			parser->setItemEnabled(importCppID,TRUE);
			exporter->setItemEnabled(exportJavaID,FALSE);
			parser->setItemEnabled(importJavaID,FALSE);
	
		}	
		else{
			exporter->setItemEnabled(exportJavaID,TRUE);
			parser->setItemEnabled(importJavaID,TRUE);
			exporter->setItemEnabled(exportCppID,FALSE);
			parser->setItemEnabled(importCppID,FALSE);
	
		}
	}
}

/**********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'entrer l'emplacement du projet  charger
**********************************************************/
void BarreDeMenu::ouvrir()
{
	QString s( QFileDialog::getOpenFileName( QString::null, "XmlFiles (*.xml)", this ) );
	if ( ! s.isEmpty() ) {
	  if(noyau->chargement(string(s))) {
		noyau->notifier();
		((FenetrePrincipale *)topLevelWidget())->mettreAJourVecteurs();
	  } else {
	    QMessageBox::warning(0,"Attention","Le chargement du fichier a chou, fichier XML incorrect");
	  }
	}
}

/**********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'entrer les infos de base sur le nouveau
 *projet
**********************************************************/
void BarreDeMenu::nouveau()
{
	FenetreCreerProjet * fen;
	fen = new FenetreCreerProjet( ((FenetrePrincipale *)topLevelWidget()) );
	fen->show();
}

/**********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur de choisir ou enregistrer le projet
**********************************************************/
void BarreDeMenu::enregistrer()
{
	Projet * p=noyau->getProjet();
	QString stringNom = QString(p->getNom().c_str());
	stringNom.replace(QRegExp(" "),"_");
	stringNom += ".xml";	
	QString fileName = QFileDialog::getSaveFileName(stringNom, "Xmlfiles (*.xml)", this );
	if ( !fileName.isNull() ) {
		noyau->sauvegarde(string(fileName));
	}
}

/**********************************************************
 * 							  *
 *							  *
**********************************************************/
void BarreDeMenu::fermer()
{
	int ret;
	FenetrePrincipale * fenPrinc = (FenetrePrincipale *)topLevelWidget();
	
	ret = QMessageBox::warning (NULL,"Fenetre Principale","Attention toutes vos donnes vont tre perdues\n\nVoulez-vous sauver le projet en cours?",
     				"Sauver","Fermer","Annuler",0,-1 );
	if (ret == 0){
		Projet * p=noyau->getProjet();
		QString stringNom = QString(p->getNom().c_str());
		stringNom.replace(QRegExp(" "),"_");
		stringNom += ".xml";	
		QString fileName = QFileDialog::getSaveFileName(stringNom, "Xmlfiles (*.xml)", this );
		if ( !fileName.isNull() ) {
			if(noyau->sauvegarde(string(fileName)) == false){
				QMessageBox::critical(0,"Fenetre Principale","La sauvegarde a chou");
				return;
			}
			fenPrinc->vctPaquetage.erase(fenPrinc->vctPaquetage.begin(),fenPrinc->vctPaquetage.end());
			fenPrinc->vctClasse.erase(fenPrinc->vctClasse.begin(),fenPrinc->vctClasse.end());
			fenPrinc->vctRelation.erase(fenPrinc->vctRelation.begin(),fenPrinc->vctRelation.end());
			delete(p);
			noyau->setProjet(NULL);
			fenPrinc->setNoeudCourant(NULL);
			noyau->notifier();
		}
		return;
	}
	else{
		if (ret == 1){
			Projet * p=noyau->getProjet();
			fenPrinc->vctPaquetage.erase(fenPrinc->vctPaquetage.begin(),fenPrinc->vctPaquetage.end());
			fenPrinc->vctClasse.erase(fenPrinc->vctClasse.begin(),fenPrinc->vctClasse.end());
			fenPrinc->vctRelation.erase(fenPrinc->vctRelation.begin(),fenPrinc->vctRelation.end());
			delete(p);
			noyau->setProjet(NULL);
			fenPrinc->setNoeudCourant(NULL);
			noyau->notifier();
			return;
		}
	}
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::informations()
{
	FenetrePrincipale * fenPrinc = (FenetrePrincipale *)topLevelWidget();
	fenPrinc->setNoeudCourant(noyau->getProjet());

	noyau->notifier();
}

/*********************************************************
 *cette fonction dtruit le topLevelWidget (la fenetre principale)
 *ainsi l'application se termine
**********************************************************/
void BarreDeMenu::quitter()
{
	delete(topLevelWidget());
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::couper()
{
	noeudEnMemoire = ((FenetrePrincipale *)topLevelWidget())->getNoeudCourant();
	couperEnMemoire = true;
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::copier()
{
	noeudEnMemoire = ((FenetrePrincipale *)topLevelWidget())->getNoeudCourant();
	couperEnMemoire = false;
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::coller()
{
	ajoutOK = false;

	if (noeudEnMemoire != NULL)
	{
		FenetrePrincipale * fen = (FenetrePrincipale *)topLevelWidget();
		VisiteurAjouter *vAjouter = new VisiteurAjouter(fen);
		noeudEnMemoire->accepter(vAjouter);
		if (couperEnMemoire && ajoutOK)
		{
			VisiteurSupprimer *vSupprimer = new VisiteurSupprimer(fen);
			noeudEnMemoire->accepter(vSupprimer);
			couperEnMemoire = false;
		}
		noyau->notifier();
	}
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::supprimer() {
	FenetrePrincipale * fen = (FenetrePrincipale *)topLevelWidget();

	Noeud * noeud = fen->getNoeudCourant();

	noeud->accepter(visiteurSupprimer);

	fen->setNoeudCourant(NULL);
	noyau->notifier();
}


/**********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'entrer ses prfrences (optionnel)
**********************************************************/
void BarreDeMenu::preferences()
{
	FenetrePreference * fen = ((FenetrePrincipale *)topLevelWidget())->getFenetrePreference();
	fen = new FenetrePreference(noyau);
	fen->show();
}

/*********************************************************
 *
 *
 *
**********************************************************/
void BarreDeMenu::vue()
{
	FenetreVue * fen ;
	fen = new FenetreVue( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'ajouter un paquetage (ventuellement
 *inclus dans un autre paquetage
**********************************************************/
void BarreDeMenu::paquetage()
{
	FenetrePaquetage * fen ;
	fen = new FenetrePaquetage( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'ajouter un commentaire (juste le texte)
 *la (ou les) destination(s) du commentaires st ds relations
**********************************************************/
void BarreDeMenu::commentaire()
{
	FenetrePrincipale *fen2= (FenetrePrincipale *)topLevelWidget();
	Noyau *n = fen2->getNoyau();
	if((n->getProjet()->getDiagrammeStatique()->NbVue()) == 0)
	{
		QMessageBox::information(0,"Fenetre Principale","Impossible d'ajouter un commentaire, il n'y a pas de vue cre");
	}
	else
	{
	FenetreCommentaire * fen ;
	fen = new FenetreCommentaire( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
	}
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'ajouter une classe dans un paquetage
 *ou en tant qu'orpheline
**********************************************************/
void BarreDeMenu::classe()
{
	FenetreClasse * fen ;
	fen = new FenetreClasse( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'ajouter une mthode  une classe
**********************************************************/
void BarreDeMenu::methode()
{
	FenetrePrincipale *fen2= (FenetrePrincipale *)topLevelWidget();
	if(fen2->vctClasse.size() == 0)
	{
		QMessageBox::information(0,"Fenetre Principale","Impossible d'ajouter une mthode, il n'y a pas de classe cre");
	}
	else
	{
	FenetreMethode * fen ;
	fen = new FenetreMethode( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
	}
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'ajouter un attribut  une classe
**********************************************************/
void BarreDeMenu::attribut()
{
	FenetrePrincipale *fen2= (FenetrePrincipale *)topLevelWidget();
	if(fen2->vctClasse.size() == 0)
	{
		QMessageBox::information(0,"Fenetre Principale","Impossible d'ajouter un attribut, il n'y a pas de classe cre");
	}
	else
	{
	FenetreAttribut * fen ;
	fen = new FenetreAttribut( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
	}
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'ajouter une relation entre deux classes
 *ou depuis un commentaire vers plusieurs destinataires
**********************************************************/
void BarreDeMenu::relation()
{
	FenetrePrincipale *fen2= (FenetrePrincipale *)topLevelWidget();
	if(fen2->vctClasse.size() < 1)
	{
		QMessageBox::information(0,"Fenetre Principale","Impossible d'ajouter une relation, il n'y a pas assez de classes");
	}
	else
	{
	FenetreRelation * fen ;
	fen = new FenetreRelation( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
	}
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::exportCpp()
{
	FenetreExport * fen ;
	fen = new FenetreExport( (FenetrePrincipale *)topLevelWidget(), EXP_CPP );
	fen->show();
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::exportJava()
{
	FenetreExport * fen ;
	fen = new FenetreExport( (FenetrePrincipale *)topLevelWidget(), EXP_JAVA);
	fen->show();
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::importCpp()
{
	FenetreParseurCpp * fen ;
	fen = new FenetreParseurCpp( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::importJava()
{
	FenetreParseurJava * fen ;
	fen = new FenetreParseurJava( (FenetrePrincipale *)topLevelWidget() );
	fen->show();
}

/*********************************************************
 *cette fonction affiche une fenetre de dialogue permettant
 *a l'utilisateur d'obtenir de l'aide sur UML
**********************************************************/
void BarreDeMenu::aide()
{
	FenetreAide *fen;
	fen = new FenetreAide("/home/iupgmi3/partage/DOCS/QT-DOC/DOC/html/index.html", ".", 0, "help viewer");
	fen->show();
}

/**********************************************************
 *
 *
**********************************************************/
void BarreDeMenu::aPropos()
{
	QMessageBox::about( this, "Gnuml v/t: Viewer / Transcoder",
			"<p><font size=+1>gnuml v/t</font> - Version: 0.1.3 - "
			"<i>http://gnuml.tuxfamily.org/</i></p>"
			"<hr>"
			"<b>Auteurs:</b><br>
			Syj - parseur/import - syj@ifrance.com<br>
			Pico  - ihm    - brownshoes@caramail.com<br>
			Gq    - ihm    - gqone@ifrance.com<br>
			Maule - ihm    - maule@caramail.com<br>
			Titux - noyau  - c.lucas@ifrance.com<br>
			Nico  - noyau  - flashingcomet@caramail.com<br>
			Jesus - export - jesus13@ifrance.com
			</p>"
			"<hr>"
			"Logiciel de modlisation UML, d'import/export d'architecture logicielle."
			"<hr>"
			"<p>Merci  Messieurs Giroux et Uppman pour leurs conseils aviss et l'intert"
			" qu'ils ont port  ce projet.</p>"
	);
}

void BarreDeMenu::ajouterDansVue( int vueID )
{
	int i;

	std::vector<Vue *> vecVues = noyau->getProjet()->getDiagrammeStatique()->getVctVue();
	std::vector<Vue *>::iterator it = vecVues.begin();
	for(i=0; i<vueID; ++i)	++it;

	FenetrePrincipale *f=(FenetrePrincipale *)topLevelWidget();
	ElementDeModelisation *edm=(ElementDeModelisation *)f->getNoeudCourant();

	if ( (*it)->ajouterElementDeModelisation(edm, 0, 0, 50, 50, TRUE) == FALSE ) {
			QMessageBox::information(0, "Erreur d'ajout", "Impossible d'ajouter cet objet  cette vue");		
	} else noyau->notifier();
}

void BarreDeMenu::lierCommentaire ( int cibleID )
{
	Commentaire * source = (Commentaire *) ((FenetrePrincipale *)topLevelWidget())->getNoeudCourant();
	Noeud * destination ;
	Classe * c ;

	std::vector<ObjetAffichable *> vecObjet = ((Vue *)source->getPere())->getvctObjetAffichable();
	std::vector<ObjetAffichable *>::iterator itObjet = vecObjet.begin();

	bool trouve = FALSE;
	int i = 0 ;
	destination = (*itObjet)->getedm() ;
	if ( destination == source ) {
		++itObjet;
		destination = (*itObjet)->getedm();
	}

	while ( i < cibleID ) {

		if ( destination != source) {
			if ( i == cibleID ) { trouve = TRUE ; break;}
			if ( ((FenetrePrincipale*)topLevelWidget())->estClasse(destination) ) {
					c = (Classe *)destination ;

					std::vector<Methode *> vctM = c->getVctMethode();
					std::vector<Methode *>::iterator itM = vctM.begin() ;
					while ( itM != vctM.end() ) {
						destination = *itM;
						if ( i == cibleID-1 ) { trouve = TRUE ; break; }
						else i++;
						++itM;
					}

					if ( !trouve ) {
						std::vector<Attribut *> vctA = c->getVctAttribut();
						std::vector<Attribut *>::iterator itA = vctA.begin() ;
						while ( itA != vctA.end() ) {
							destination = *itA;
							if ( i == cibleID-1 ) { trouve = TRUE ; break ; }
							else i++;
							++itA;
						}
					}
			}
			++i;
		}
		++itObjet;
		if ( !trouve ) destination = (*itObjet)->getedm();
	}

	source->ajouterRelationCommentaire( "commente \""+destination->getNom()+"\"", destination );
	noyau->notifier();
}


