#ifndef BARREOUTILSAJOUTER_H
#define BARREOUTILSAJOUTER_H

#include <qtoolbar.h>

#include <noyobservateur.h>
#include <noynoyau.h>

/** Cette classe represente une barre d'outils qui reprend les optionnalits du
 * menu Edition/Ajouter de la barre de menu.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class BarreOutilsAjouter : public QToolBar, public Observateur
{
	Q_OBJECT

	public:
		/** Constructeur
		 * \param Noyau *n : Represente le noyau associ au projet.
		 * \param QWidget *parent : Represente le widget parent de cet barre d'outils (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  notre barre d'outils (Par defaut 0).
		 */
		BarreOutilsAjouter(Noyau * n, QMainWindow *parent=0, const char *name=0);

		//! Destructeur
		virtual ~BarreOutilsAjouter() {}


		/** Cette methode averti la barre d'outils qu'il y a eu un changement dans le noyau.
		 * Elle provoque le raffraichissement de la barre d'outils.
		 */
		void avertir();

		/** Permet de recuperer le noyau qu'coute cet barre d'outils.
		 * \return Une reference sur le noyau qu'coute cet barre d'outils.
		 */
		Noyau * getNoyau() { return noyau ; }		


		/** Permet de recuperer une reference sur le bouton d'ajout d'une vue.
		 * \return Une reference sur le bouton d'ajout d'une vue.
		 */
		QToolButton * getBoutonVue();

		/** Permet de recuperer une reference sur le bouton d'ajout d'un paquetage.
		 * \return Une reference sur le bouton d'ajout d'un paquetage.
		 */
		QToolButton * getBoutonPaquetage();

		/** Permet de recuperer une reference sur le bouton d'ajout d'un commentaire.
		 * \return Une reference sur le bouton d'ajout d'un commentaire.
		 */
		QToolButton * getBoutonCommentaire();

		/** Permet de recuperer une reference sur le bouton d'ajout d'une classe.
		 * \return Une reference sur le bouton d'ajout d'une classe.
		 */
		QToolButton * getBoutonClasse();

		/** Permet de recuperer une reference sur le bouton d'ajout d'une methode.
		 * \return Une reference sur le bouton d'ajout d'une methode.
		 */
		QToolButton * getBoutonMethode();

		/** Permet de recuperer une reference sur le bouton d'ajout d'un attribut.
		 * \return Une reference sur le bouton d'ajout d'un attribut.
		 */
		QToolButton * getBoutonAttribut();

		/** Permet de recuperer une reference sur le bouton d'ajout d'une relation.
		 * \return Une reference sur le bouton d'ajout d'une relation.
		 */
		QToolButton * getBoutonRelation();

		/** Permet de recuperer une reference sur le bouton de lancement de la
		 * procedure d'export en C++.
		 * \return Une reference sur le bouton de lancement de la procedure d'export en C++.
		 */
		QToolButton * getBoutonExportCpp();

		/** Permet de recuperer une reference sur le bouton de lancement de la
		 * procedure d'export en Java.
		 * \return Une reference sur le bouton de lancement de la procedure d'export en Java.
		 */
		QToolButton * getBoutonExportJava();

		/** Permet de recuperer une reference sur le bouton d'ajout d'une vue.
		 * \return Une reference sur le bouton d'ajout d'une vue.
		 */
		QToolButton * getBoutonImportCpp();

		/** Permet de recuperer une reference sur le bouton d'ajout d'une vue.
		 * \return Une reference sur le bouton d'ajout d'une vue.
		 */
		QToolButton * getBoutonImportJava();

	private:
		Noyau * noyau ;
		QToolButton *boutonVue, *boutonPaquetage, *boutonCommentaire, *boutonClasse, *boutonMethode, *boutonAttribut, *boutonRelation ;
		QToolButton *boutonExportCpp, *boutonExportJava, *boutonImportCpp, *boutonImportJava ;
};

#endif

