#ifndef BARREOUTILSEDITION_H
#define BARREOUTILSEDITION_H

#include <qtoolbar.h>

#include <noyobservateur.h>
#include <noynoyau.h>

/** Cette classe represente une barre d'outils qui reprend certaines optionnalits du
 * menu Edition de la barre de menu.
 * @author : F.Duchaussoy, N.Longuet, G.Prevot
 */

class BarreOutilsEdition : public QToolBar, public Observateur
{
	Q_OBJECT

	public:
		/** Constructeur
		 * \param Noyau *n : Represente le noyau associ au projet.
		 * \param QWidget *parent : Represente le widget parent de cet barre d'outils (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  notre barre d'outils (Par defaut 0).
		 */ 
		BarreOutilsEdition(Noyau * n, QMainWindow *parent=0, const char *name=0);

		//! Destructeur
		virtual ~BarreOutilsEdition() {}


		/** Cette methode averti la barre d'outils qu'il y a eu un changement dans le noyau.
		 * Elle provoque le raffraichissement de la barre d'outils.
		 */
		void avertir();

		/** Permet de recuperer le noyau qu'coute cet barre d'outils.
		 * \return Une reference sur le noyau qu'coute cet barre d'outils.
		 */
		Noyau * getNoyau() { return noyau ; }


		/** Permet de recuperer une reference sur le bouton couper de notre barre d'outils.
		 * \return Une reference sur le bouton couper de notre barre d'outils.
		 */
		QToolButton * getBoutonCouper();

		/** Permet de recuperer une reference sur le bouton copier de notre barre d'outils.
		 * \return Une reference sur le bouton copier de notre barre d'outils.
		 */
		QToolButton * getBoutonCopier();

		/** Permet de recuperer une reference sur le bouton coller de notre barre d'outils.
		 * \return Une reference sur le bouton coller de notre barre d'outils.
		 */
		QToolButton * getBoutonColler();

		/** Permet de recuperer une reference sur le bouton supprimer de notre barre d'outils.
		 * \return Une reference sur le bouton recuperer de notre barre d'outils;
		 */
		QToolButton * getBoutonSupprimer();


	private:
		Noyau * noyau;
		QToolButton *boutonCouper, *boutonCopier, *boutonColler, *boutonSupprimer;
};

#endif

