#ifndef BARREOUTILSPROJET_H
#define BARREOUTILSPROJET_H

#include <qtoolbar.h>

#include <noyobservateur.h>
#include <noynoyau.h>

/** Cette classe represente une barre d'outils qui reprend certaines optionnalits du
 * menu Projet de la barre de menu.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class BarreOutilsProjet : public QToolBar, public Observateur
{
	Q_OBJECT

	public:

		/** Constructeur
		 * \param Noyau *n : Represente le noyau associ au projet.
		 * \param QWidget *parent : Represente le widget parent de cet barre d'outils (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  notre barre d'outils (Par defaut 0).
		 */
		BarreOutilsProjet(Noyau * n, QMainWindow *parent=0, const char *name=0);

		//! Destructeur
		virtual ~BarreOutilsProjet() {}

		/** Cette methode averti la barre d'outils qu'il y a eu un changement dans le noyau.
		 * Elle provoque le raffraichissement de la barre d'outils.
		 */
		void avertir();

		/** Permet de recuperer le noyau qu'coute cet barre d'outils.
		 * \return Une reference sur le noyau qu'coute cet barre d'outils.
		 */
		Noyau * getNoyau() { return noyau ; }


		/** Permet de recuperer une reference sur le bouton ouvrir de notre barre d'outils.
		 * \return Une reference sur le bouton ouvrir de notre barre d'outils.
		 */
		QToolButton * getBoutonOuvrir();

		/** Permet de recuperer une reference du bouton nouveau de notre barre d'outils.
		 * \return Une reference sur le bouton nouveau de notre barre d'outils.
		 */
		QToolButton * getBoutonNouveau();

		/** Permet de recuperer une reference sur le bouton enregistrer de notre barre d'outils.
		 * \return Une reference sur le bouton enregistrer de notre barre d'outils.
		 */
		QToolButton * getBoutonEnregistrer();

		/** Permet de recuperer une reference sur le bouton informations de notre barre d'outils.
		 * \return Une reference sur le bouton informations de notre barre d'outils.
		 */
		QToolButton * getBoutonInformations();

	private:
		Noyau * noyau;
		QToolButton *boutonOuvrir, *boutonNouveau, *boutonEnregistrer, *boutonInformations ;
};

#endif

