#include <ihmfenetreaide.h>

#include <qstatusbar.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qiconset.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstylesheet.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qapplication.h>
#include <qcombobox.h>
#include <qevent.h>
#include <qlineedit.h>
#include <qobjectlist.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qprinter.h>
#include <qsimplerichtext.h>
#include <qpaintdevicemetrics.h>

#include <qmessagebox.h>

#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <iostream.h>

FenetreAide::FenetreAide( const QString& home_, const QString& _path,
			QWidget* parent, const char *nom )
    : QMainWindow( parent, nom, WDestructiveClose ),
      pathCombo( 0 ), selectedURL()
{
    browser = new QTextBrowser( this );
    browser->mimeSourceFactory()->setFilePath( _path );
    browser->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    connect( browser, SIGNAL( textChanged() ),
	     this, SLOT( textChanged() ) );

    setCentralWidget( browser );

    if ( !home_.isEmpty() )
	browser->setSource( home_ );

    connect( browser, SIGNAL( highlighted( const QString&) ),
	     statusBar(), SLOT( message( const QString&)) );

    resize( 640,700 );

    QPopupMenu* file = new QPopupMenu( this );
    file->insertItem( tr("&Ouvrir un fichier"), this, SLOT( openFile() ), ALT | Key_O );
    file->insertSeparator();
    file->insertItem( tr("&Sortir"), this, SLOT( close() ), ALT | Key_S );

    // The same three icons are used twice each.
    QIconSet icon_back( QPixmap("./xpms/back.xpm") );
    QIconSet icon_forward( QPixmap("./xpms/forward.xpm") );
    QIconSet icon_home( QPixmap("./xpms/home.xpm") );

    QPopupMenu* go = new QPopupMenu( this );
    backwardId = go->insertItem( icon_back,
				 tr("&Backward"), browser, SLOT( backward() ),
				 ALT | Key_Left );
    forwardId = go->insertItem( icon_forward,
				tr("&Forward"), browser, SLOT( forward() ),
				ALT | Key_Right );
    go->insertItem( icon_home, tr("&Home"), browser, SLOT( home() ) );

    bookm = new QPopupMenu( this );
    bookm->insertItem( tr( "Aide sur GNUML" ), this, SLOT( voirAideGNUML() ) );
    bookm->insertItem( tr( "Aide sur l'UML" ), this, SLOT( voirAideUML() ) );
    bookm->insertItem( tr( "site gnuml" ), this, SLOT( voirAideSITE() ) );
    bookm->insertItem( tr( "A propos..." ), this, SLOT( voirAPropos() ) );

    menuBar()->insertItem( tr("&Fichier"), file );
    menuBar()->insertItem( tr("&Aller"), go );
    menuBar()->insertItem( tr( "Aide" ), bookm );

    menuBar()->setItemEnabled( forwardId, FALSE);
    menuBar()->setItemEnabled( backwardId, FALSE);
    connect( browser, SIGNAL( backwardAvailable( bool ) ),
	     this, SLOT( setBackwardAvailable( bool ) ) );
    connect( browser, SIGNAL( forwardAvailable( bool ) ),
	     this, SLOT( setForwardAvailable( bool ) ) );


    QToolBar* toolbar = new QToolBar( this );
    addToolBar( toolbar, "Toolbar");
    QToolButton* button;

    button = new QToolButton( icon_back, tr("Backward"), "", browser, SLOT(backward()), toolbar );
    connect( browser, SIGNAL( backwardAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
    button = new QToolButton( icon_forward, tr("Forward"), "", browser, SLOT(forward()), toolbar );
    connect( browser, SIGNAL( forwardAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
    button = new QToolButton( icon_home, tr("Home"), "", browser, SLOT(home()), toolbar );

    toolbar->addSeparator();

    pathCombo = new QComboBox( TRUE, toolbar );
    toolbar->setStretchableWidget( pathCombo );
    setRightJustification( TRUE );
    setDockEnabled( Left, FALSE );
    setDockEnabled( Right, FALSE );

    pathCombo->insertItem( home_ );
	pathCombo->insertItem( "a.html" );
	pathCombo->insertItem( "lm.html" );

    browser->setFocus();
}


void FenetreAide::setBackwardAvailable( bool b)
{
    menuBar()->setItemEnabled( backwardId, b);
}

void FenetreAide::setForwardAvailable( bool b)
{
    menuBar()->setItemEnabled( forwardId, b);
}


void FenetreAide::textChanged()
{
cout<< "HelpWidow::textChanged()" << endl ;
    if ( browser->documentTitle().isNull() )
	setCaption( browser->context() );
    else
	setCaption( browser->documentTitle() ) ;

    selectedURL = caption();
    if ( !selectedURL.isEmpty() && pathCombo ) {
	bool exists = FALSE;
	int i; 
	for ( i = 0; i < pathCombo->count(); ++i ) {
	    if ( pathCombo->text( i ) == selectedURL ) {
		exists = TRUE;
		break;
	    }
	}
	if ( !exists ) {
	    pathCombo->insertItem( selectedURL, 0 );
	    pathCombo->setCurrentItem( 0 );
	} else
	    pathCombo->setCurrentItem( i );
	selectedURL = QString::null;
    }
}

FenetreAide::~FenetreAide()
{
}

void FenetreAide::openFile()
{
#ifndef QT_NO_FILEDIALOG
	QString fn = QFileDialog::getOpenFileName( QString::null, "*.html", this );
	if ( !fn.isEmpty() ) {
		browser->mimeSourceFactory()->setFilePath( fn );
		browser->setSource( fn );
	}
#endif
}

void FenetreAide::voirAideGNUML()
{
	char		t[255];
	
	getcwd( t , 255 );
	strcat( t , "/lm.html" );
	browser->mimeSourceFactory()->setFilePath( t );
	browser->setSource( t );
}

void FenetreAide::voirAideUML()
{
	char	t[255];
	
	getcwd( t , 255 );
	strcat( t , "/a.html" );
	browser->mimeSourceFactory()->setFilePath( t );
	browser->setSource( t );
}

void FenetreAide::voirAideSITE()
{
	browser->setText("");
	const QString & q = "http://gnuml.tuxfamily.org" ;
	browser->setSource( q );
}

void FenetreAide::voirAPropos()
{
 QMessageBox::about( this, "Gnuml v/t: Viewer / Transcoder",
			"<p><font size=+1>gnuml v/t</font> - Version: 0.1.3 - "
			"<i>http://gnuml.tuxfamily.org/</i></p>"
			"<hr>"
			"<b>Auteurs:</b><br>
			Syj - parseur/import - syj@ifrance.com<br>
			Pico  - ihm    - brownshoes@caramail.com<br>
			Gq    - ihm    - gqone@ifrance.com<br>
			Maule - ihm    - maule@caramail.com<br>
			Titux - noyau  - c.lucas@ifrance.com<br>
			Nico  - noyau  - flashingcomet@caramail.com<br>
			Jesus - export - jesus13@ifrance.com
			</p>"
			"<hr>"
			"Logiciel de modlisation UML, d'import/export d'architecture logicielle."
			"<hr>"
			"<p>Merci  Messieurs Giroux et Uppman pour leurs conseils aviss et l'intert"
			" qu'ils ont port  ce projet.</p>"
			);

}
