#ifndef FENETREAIDE_H
#define FENETREAIDE_H

#include <qmainwindow.h>
#include <qtextbrowser.h>
#include <qstringlist.h>
#include <qdir.h>

class QComboBox;
class QPopupMenu;
class QMessageBox ;

/** Cette classe represente la fentre d'aide qui contiendra une barre de menu,
 * une barre d'outils et un QTextBrowser. C'est en fait un mini-navigateur qui
 * permettra de naviguer dans les pages d'aide.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class FenetreAide : public QMainWindow
{
	Q_OBJECT
public:
	/** Constructeur
	 * \param const QString& home_ : Endroit o se trouve la page home
	 * \param const QString& path : Chemin du rpertoire
	 * \param QWidget* parent : Represente le widget parent de cette fentre (Par defaut 0).
	 * \param const char *nom : Represente le nom que l'on veut donner  cette fentre (Par defaut 0).
	 */
	FenetreAide( const QString& home_,  const QString& path, QWidget* parent = 0, const char *nom=0 );

	//! Destructeur
	~FenetreAide();

private slots:
	/** Ce slot active ou desactive le bouton backward de la barre d'outils.
	 * \param bool : Rend le bouton utilisable si TRUE, inutilisable si FALSE.
	 */
	void setBackwardAvailable( bool );

	/** Ce slot active ou desactive le bouton forward de la barre d'outils.
	 * \param bool : Rend le bouton utilisable si TRUE, inutilisable si FALSE.
	 */
	void setForwardAvailable( bool );	

	/** Ce slot reagit au changement de texte dans le navigateur en memorisant
	 * la derniere adresse dans l'historique.
	 */
	void textChanged( );

	//!  Ce slot lance la procedure d'ouverture d'une page dans le navigateur.
	void openFile( );

	//! Ce slot affiche la page d'aide sur UML v/t dans le navigateur.
	void voirAideGNUML( );

	//! Ce slot affiche la page d'aide sur l'UML dans le navigateur.
	void voirAideUML( );

	//! Ce slot affiche la page du site internet dans le navigateur.
	void voirAideSITE( );

	//! Ce slot affiche la fentre  propos du logiciel.
	void voirAPropos( );

private:
    
	QTextBrowser	*browser;
	QComboBox 		*pathCombo;
	int 		backwardId, forwardId;
	QString 		selectedURL;
	QPopupMenu 		*hist, *bookm;
    
};

#endif
