#define INTERFACE false
#define CLASSE true

#include <qwidget.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qmessagebox.h>

#include <ihmfenetreclasse.h>

FenetreClasse::FenetreClasse(FenetrePrincipale * fen, Classe * c, QWidget *parent, const char *name)
	: QWidget(parent,name)
{
	fenetre = fen;
	classe = c;

	setCaption("Ajouter une classe");

	/* Zone d'entre de nom */	
	QGroupBox *groupNom = new QGroupBox (1, Vertical,"Nom", this, "groupNom" );
	editNom = new QLineEdit (groupNom, "editNom"); 

	/* Zone d'appartenance  un paquetage */
	QGroupBox *groupPaquetage = new QGroupBox (1, Vertical,"Paquetage Pre", this, "groupPaquetage");
	if ( classe == NULL ) 
		if (fen->estPaquetage(fen->getNoeudCourant())) comboPaquetage = new MonComboPaquetage ( (Paquetage *)fen->getNoeudCourant(), fenetre, groupPaquetage, "comboPaquetage" );
		else comboPaquetage = new MonComboPaquetage ( NULL, fenetre, groupPaquetage, "comboPaquetage" );
	else comboPaquetage = new MonComboPaquetage ( (Paquetage *)c->getPere(), fenetre, groupPaquetage, "comboPaquetage" );

	/* Zone de visibilit */
	QGroupBox *groupVisibilite = new QGroupBox (1, Vertical,"Visibilit", this, "groupVisibilite");
	comboVisibilite = new QComboBox (groupVisibilite, "comboVisibilite");
	comboVisibilite->insertItem ( "public", 0 );
	comboVisibilite->insertItem ( "protected", 1 );
	comboVisibilite->insertItem ( "private", 2 );

	QGroupBox *groupDescription = new QGroupBox(1, Vertical, "Description", this, "groupDescription");
	editDescription = new QMultiLineEdit(groupDescription , "editDescription");
	checkInterface = new QCheckBox ( "Interface", groupDescription, "checkInterface" ); 

	/* Grille pour placer les lments de la fentre */
	QGridLayout *grille = new QGridLayout( this, 9, 1, 1 );
	QGridLayout *bouton = new QGridLayout( 1, 3, 1);
 
	/* placement des lments */
	grille->addWidget(groupNom, 5, 0);
	grille->addWidget(groupPaquetage, 3, 0);
	grille->addWidget(groupVisibilite, 1, 0);
	grille->addWidget(groupDescription, 7, 0);
	grille->addLayout(bouton, 9, 0);

	QPushButton *ok = new QPushButton ( "&Ok", this, "ok");
	connect( ok, SIGNAL(clicked()), this, SLOT(ok()));
	bouton->addWidget(ok, 0, 1);

	if ( classe == NULL ) {
		setGeometry(100,100,400,300);
		QPushButton *annuler = new QPushButton ( "&Annuler", this, "annuler");
		connect( annuler, SIGNAL(clicked()), this, SLOT(annuler()));
		bouton->addWidget(annuler, 0, 2);
	} else {
		if ( parentWidget() == NULL ) setMinimumSize( 400 , 300 );
		else setMinimumSize(parentWidget()->size());
		
		adjustSize();

		if ( QString(classe->getVisibilite().c_str()).compare("public") == 0 )
			comboVisibilite->setCurrentItem(0);
		else if ( QString(classe->getVisibilite().c_str()).compare("protected") == 0)
			comboVisibilite->setCurrentItem(1);
		else
			comboVisibilite->setCurrentItem(2);

		editNom->setText( QString(classe->getNom().c_str()) );
		editDescription->setText( QString(classe->getDescription().c_str()) ); 
		if ( classe->getTypeDeClasse() == INTERFACE ) checkInterface->setChecked( true );
		else checkInterface->setChecked( false );
	}
}

void FenetreClasse::ok()
{
	QString nom = editNom->text().replace(QRegExp(" "),"");

	if ( nom == "" ) {
		QMessageBox::warning(0,"Fenetre classe","Impossible d'ajouter une classe dont le nom est vide");
		return;
	}

	nom[0] = nom[0].upper();
	editNom->setText(nom);

	Noyau * n = fenetre->getNoyau();
	Projet * p = n->getProjet();
	DiagrammeStatique * d = p->getDiagrammeStatique();

	bool err ;

	Noeud *pere_nouveau ;
	int indice_pere_nouveau = comboPaquetage->currentItem();

	if ( indice_pere_nouveau == 0 ) pere_nouveau = d;
	else {
		std::vector<Paquetage*>::iterator it = fenetre->vctPaquetage.begin();
		int i = 1;
		while( i<indice_pere_nouveau ){
			++it;
			++i;
		}
		pere_nouveau = *it ;
	}

	if ( classe == NULL ) {
		if (string(comboVisibilite->currentText()) != "public")
		if (checkInterface->isChecked() == true){
			QMessageBox::warning(0,"Fenetre classe","Une interface doit tre publique");
			comboVisibilite->setCurrentItem(0);
			return;
		}		

		Classe * cl = new Classe( pere_nouveau, string(editNom->text()) , 
				string(comboVisibilite->currentText()) , !checkInterface->isChecked() );
		
		if ( indice_pere_nouveau == 0 ) err = ((DiagrammeStatique *)pere_nouveau)->ajouterObjetuml( (Objetuml *)cl );
		else err = ((Paquetage *)pere_nouveau)->ajouterObjetuml( (Objetuml *)cl );
	
		if ( err == FALSE ) {
			QMessageBox::information(0,"Fenetre classe","Impossible d'ajouter une classe avec ces paramtres");
		} else {
			if ( err=((Noeud *)cl)->setNom( string(editNom->text()) ) == FALSE )
					QMessageBox::information(0,"Fenetre classe","Impossible de donner ce nom  une classe");
			else ((Noeud *)cl)->setDescription( string(editDescription->text()) );

			cout << "AJOUT (classe) " << cl << " dans " << pere_nouveau << endl;
			fenetre->ajouterClasse(cl);
			std::vector<Vue*> v = fenetre->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
			std::vector<Vue*>::iterator itv;
			Noeud * no = fenetre->getNoeudCourant();
			itv = v.begin();
			while (itv != v.end()){
				if ((*itv) == no){
					(*itv)->ajouterElementDeModelisation((ElementDeModelisation*)cl, 20 , 20 , 20 , 20 , true);
					break;
				}
				++itv;
			}

			delete(this);
		}
	} else {
		int ret;
		if ( checkInterface->isChecked() == true){
			ret = QMessageBox::warning (NULL,"Fenetre CLasse","Attention mettre une classe en interface supprimera le code des methodes de celle-ci",
     				"Valider","Annuler",NULL,0,-1 );
			if (ret == 1) return;
		}
		if ( editNom->text() == "" )
				QMessageBox::warning(0,"Fenetre classe","Impossible de renommer une classe avec un nom vide");
		else {
			if ( ((Noeud *)classe)->setNom( string(editNom->text()) ) == FALSE )
				QMessageBox::warning(0,"Fenetre classe","Impossible de renommer une classe avec ce nom");
			else {
				if (string(comboVisibilite->currentText()) != "public")
					if (checkInterface->isChecked() == true){
						QMessageBox::warning(0,"Fenetre classe","Une interface doit tre publique");
						comboVisibilite->setCurrentItem(0);
						return;
					}	
				
				std::vector<Methode *> vm;
				std::vector<Methode *>::iterator itm;
				vm = classe->getVctMethode();
				itm = vm.begin();
				while(itm != vm.end()){
					if ((*itm)->getTypeRetour() == "") (*itm)->setNom(string(editNom->text()));
					++itm;
				}
	
				classe->setVisibilite( string( comboVisibilite->currentText() ) );
				((Noeud *)classe)->setDescription( string(editDescription->text()) );
				classe->setTypeDeClasse( !checkInterface->isChecked() );
				
				Noeud * pere_actuel = ((Noeud *)classe)->getPere();
				if ( pere_actuel != pere_nouveau ) {
					cout << "CHANGEMENT (classe) de " << pere_actuel << "  " << pere_nouveau << endl;
					if ( indice_pere_nouveau == 0 ) 
						((Objetuml *)classe)->deplacerVersDiagrammeStatique( pere_nouveau );
					else 
						((Objetuml *)classe)->deplacerVersPaquetage( (Paquetage *)pere_nouveau );
				}
			}
		}	        			    			
		if ( parentWidget() == NULL ) delete(this);
	}
n->notifier();
}

void FenetreClasse::annuler()
{
	delete(this);
}


