#ifndef FENETREDEDESSIN_H
#define FENETREDEDESSIN_H

#include <qframe.h>

#include <noyobservateur.h>
#include <noynoyau.h>
#include <ihmvisiteurdessin.h>

/** Cette classe represente la zone de dessin
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */
class FenetreDeDessin : public QFrame, public Observateur
{
	Q_OBJECT

	public:
		/** Constructeur
		 * \param Noyau * n : Reference sur le noyau associ  l'application.
		 * \param QWidget *parent : Represente le widget parent de la zone de dessin (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  la zone de dessin.
		 */
		FenetreDeDessin(Noyau * n, QWidget *parent=0, const char *name=0);

		//! Destructeur
		virtual ~FenetreDeDessin();

		//! Cette methode est appel lorsqu'il y a un changement dans le noyau. Elle permet de raffraichir la zone de dessin.
		void avertir();


		/** Permet de recuperer une reference sur le noyau associ  l'application.
		 * \return Une reference sur le noyau associ  l'application.
		 */
		Noyau * getNoyau();

		VisiteurDessin * getVisiteur(); 

		/** Permet de positionner le conteneur de la zone de dessin.
		 * \param QWidget *w : Widget representant le nouveau conteneur de la zone de dessin.
		 */
		void setConteneur ( QWidget * w );

		QWidget * getConteneur ( void );

	private:
		Noyau * noyau;
		VisiteurDessin * visiteur ;
		QWidget * conteneur ;
};

#endif

