#include <vector.h>

#include <qwidget.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qmessagebox.h>

#include <ihmfenetrepaquetage.h>

FenetrePaquetage::FenetrePaquetage(FenetrePrincipale * fen, Paquetage * p,
	QWidget *parent, const char *name) : QWidget(parent,name)
{
	fenetre = fen;
	paquetage = p;

	setCaption("Ajouter un paquetage");

	/* Zone d'entre de nom */	
	QGroupBox *groupNom = new QGroupBox (1, Vertical,"Nom", this, "groupNom" );
	editNom = new QLineEdit (groupNom, "editNom"); 

	/* Zone d'appartenance  un paquetage */
	QGroupBox *groupPaquetage = new QGroupBox (1, Vertical,"Paquetage Pre", this, "groupPaquetage");
	if ( p == NULL ) 
		if (fen->estPaquetage(fen->getNoeudCourant()))
			comboPaquetage = new MonComboPaquetage ( (Paquetage *)fen->getNoeudCourant(), fenetre, groupPaquetage , "comboPaquetage" );
		else 	comboPaquetage = new MonComboPaquetage ( NULL, fenetre, groupPaquetage , "comboPaquetage" );
	else comboPaquetage = new MonComboPaquetage ( (Paquetage *)p->getPere(), fenetre, groupPaquetage , "comboPaquetage" );

	/* Description */
	QGroupBox *groupDescription = new QGroupBox (1, Vertical,"Description", this, "groupDescription" );
	editDescription = new QMultiLineEdit(groupDescription , "editDescription");

	/* Grille pour placer les lments de la fentre */
	QGridLayout *grille = new QGridLayout( this, 7, 1, 1 );
	QGridLayout *bouton = new QGridLayout( 1, 3, 1);
 
	/* placement des lments */
	grille->addWidget(groupPaquetage, 1, 0);
	grille->addWidget(groupNom, 3, 0);
	grille->addWidget(groupDescription, 5, 0);
	grille->addLayout(bouton, 7, 0);

	/* Boutons */
	QPushButton *ok = new QPushButton ( "&Ok", this, "ok");
	connect( ok, SIGNAL(clicked()), this, SLOT(ok()));
	bouton->addWidget(ok, 0, 1);

	if ( paquetage == NULL ) {
		setGeometry(100,100,400,300);
		QPushButton *annuler = new QPushButton ( "&Annuler", this, "annuler");
		bouton->addWidget(annuler, 0, 2);
		connect( annuler, SIGNAL(clicked()), this, SLOT(annuler()));
	} else {
		if ( parentWidget() == NULL ) setMinimumSize( 400 , 300 );
		else setMinimumSize(parentWidget()->size());
		
		adjustSize();

		editNom->setText( QString(p->getNom().c_str()) );
		editDescription->setText( QString(p->getDescription().c_str()) ); 
	}
}

void FenetrePaquetage::ok()
{
	QString nom = editNom->text().replace(QRegExp(" "),"");

	if ( nom == "" ) {
		QMessageBox::information(0,"Ajouter un paquetage","Un paquetage ne doit pas avoir un nom vide");
		return;
	}

	// passage de la premire lettre du nom en majuscule 
	nom[0] = nom[0].upper();
	editNom->setText(nom);

	Noyau * n = fenetre->getNoyau();
	Projet * p = n->getProjet();
	DiagrammeStatique * d = p->getDiagrammeStatique();

	Noeud *pere_nouveau ;
	int indice_pere_nouveau = comboPaquetage->currentItem();

	if ( indice_pere_nouveau == 0 ) pere_nouveau = d;
	else {
		std::vector<Paquetage*>::iterator it = fenetre->vctPaquetage.begin();
		int i = 1;
		while( i<indice_pere_nouveau ){
			++it;
			++i;
		}
		pere_nouveau = *it ;
	}

	if ( paquetage == NULL ) {
		Paquetage * pa = new Paquetage( pere_nouveau, string(editNom->text()) );
		bool err ;

		if ( indice_pere_nouveau == 0 ) err = ((DiagrammeStatique *)pere_nouveau)->ajouterObjetuml( (Objetuml *)pa );
		else err = ((Paquetage *)pere_nouveau)->ajouterObjetuml( (Objetuml *)pa );

		if ( err == FALSE ) {
			QMessageBox::information(0,"Ajouter un paquetage","Impossible d'ajouter un paquetage ici");
			return;
		}
		else {
			((Noeud *)pa)->setDescription( string(editDescription->text()) );
			if ( ((Noeud *)pa)->setNom( string(editNom->text()) ) == FALSE ) {
					QMessageBox::information(0,"Ajouter un paquetage","Impossible de donner ce nom  un paquetage");
					delete(pa);
					return;
			} else {
				cout << "AJOUT (paquetage) " << pa << " dans " << pere_nouveau << endl;
				fenetre->ajouterPaquetage(pa);
				std::vector<Vue*> v = fenetre->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
				std::vector<Vue*>::iterator itv;
				Noeud * no = fenetre->getNoeudCourant();
				itv = v.begin();
				while (itv != v.end()){
					if ((*itv) == no){
						(*itv)->ajouterElementDeModelisation((ElementDeModelisation*)pa, 20 , 20 , 20 , 20 , true);
						break;
					}
					++itv;
				}	
			}
		}
		delete(this);
	} else {
		((Noeud *)paquetage)->setDescription( string(editDescription->text()) );

		if ( ((Noeud *)paquetage)->setNom( string(editNom->text()) ) == FALSE )
			QMessageBox::information(0,"Ajouter un paquetage","Impossible de donner ce nom  ce paquetage");

		Noeud * pere_actuel = ((Noeud *)paquetage)->getPere();
		if ( pere_actuel != pere_nouveau ) {

				cout << "CHANGEMENT (paquetage) de " << pere_actuel << "  " << pere_nouveau << endl;
				if ( pere_nouveau == d ) {
						((Objetuml *)paquetage)->deplacerVersDiagrammeStatique( pere_nouveau );
				} else {
						((Objetuml *)paquetage)->deplacerVersPaquetage( (Paquetage *)pere_nouveau );
				}
		}
		if ( parentWidget() == NULL ) delete(this);
	}
	n->notifier();
}

void FenetrePaquetage::annuler()
{
	delete(this);
}

