#include <qmainwindow.h>
#include <qapplication.h>
#include <qplatinumstyle.h>

#include <ihmfenetreprincipale.h>
#include <ihmvisiteurvecteur.h>

FenetrePrincipale::FenetrePrincipale(Noyau * n, QWidget *parent, const char *name)
	: QMainWindow(parent,name)
{
	noyau = n;

	// Barre de menu
	barreDeMenu = new BarreDeMenu(noyau, this);

	// Barre d'outils Ajouter
	barreOutilsAjouter = new BarreOutilsAjouter(noyau, this);
	addToolBar(barreOutilsAjouter,Right, FALSE);

	// Barre d'outils Edition
	barreOutilsEdition = new BarreOutilsEdition(noyau, this);
	addToolBar(barreOutilsEdition,Right,FALSE);

	// Barre d'outils Projet
	barreOutilsProjet = new BarreOutilsProjet(noyau, this);
	addToolBar(barreOutilsProjet,Top,FALSE);

	// Widget central
	widgetCentral = new WidgetCentral(noyau, Horizontal, this);
	setCentralWidget(widgetCentral);

	// Fenetre de dialogue
	fenetrePreference = NULL;
	navigateur = NULL;

	noeud = NULL;
	noyau->notifier();
}

FenetrePrincipale::~FenetrePrincipale()
{
	cout << "FenetrePrincipale en destruction" << endl;
}

void FenetrePrincipale::setNoeudCourant( Noeud * n )
{
	noeud = n ;
}

void FenetrePrincipale::ajouterPaquetage( Paquetage * p )
{
	vctPaquetage.push_back(p);	
}

void FenetrePrincipale::supprimerPaquetage( Paquetage * p )
{

	std::vector<Paquetage*>::iterator it;
	it = vctPaquetage.begin();
	while((it != vctPaquetage.end())&&(*it != p)){
		++it;
	}
	if (it != vctPaquetage.end()) {
		std::vector<Objetuml*> vctObjetuml = p->getVctObjetuml();
		std::vector<Objetuml*>::iterator itObjetuml = vctObjetuml.begin();
		while( itObjetuml != vctObjetuml.end() ) {
			if ( estPaquetage((Paquetage*)*itObjetuml) ) supprimerPaquetage((Paquetage*)*itObjetuml);
			else supprimerClasse((Classe*)*itObjetuml);
			++itObjetuml;
		}

		// On retire l'objet du vecteur de paquetages
		vctPaquetage.erase(it);
	}
}

bool FenetrePrincipale::estPaquetage( Noeud * n )
{
	std::vector<Paquetage*>::iterator it;
	it = vctPaquetage.begin();
	while (it != vctPaquetage.end()) {
		if ( *it == n ) return TRUE;
		++it;
	}
	return FALSE;
}

void FenetrePrincipale::ajouterClasse( Classe * c )
{
	vctClasse.push_back(c);	
}

void FenetrePrincipale::supprimerClasse( Classe * c )
{
	std::vector<Classe*>::iterator it;
	it = vctClasse.begin();
	while((it != vctClasse.end())&&(*it != c)){
		++it;
	}
	if (it != vctClasse.end()) {
		std::vector<Relation*> vctRelation = c->getVctRelation();
		std::vector<Relation*>::iterator itRelation = vctRelation.begin();
		while( itRelation != vctRelation.end() ) {
				supprimerRelation((Relation*)*itRelation);
				++itRelation;
		}

		// On retire l'objet du vecteur de classes
		vctClasse.erase(it);
	}
}

bool FenetrePrincipale::estClasse( Noeud * n )
{
	std::vector<Classe*>::iterator it;
	it = vctClasse.begin();
	while (it != vctClasse.end()) {
		if ( *it == n ) return TRUE;
		++it;
	}
	return FALSE;
}

void FenetrePrincipale::ajouterRelation( Relation * r )
{
	vctRelation.push_back(r);	
}

void FenetrePrincipale::supprimerRelation( Relation * r )
{
	std::vector<Relation*>::iterator it;
	it = vctRelation.begin();
	while((it != vctRelation.end())&&(*it != r)){
		++it;
	}
	if (it != vctRelation.end()) {
		// On retire l'objet du vecteur de relation
		vctRelation.erase(it);
	}
}

bool FenetrePrincipale::estRelation( Noeud * n )
{
	std::vector<Relation*>::iterator it;
	it = vctRelation.begin();
	while (it != vctRelation.end()) {
		if ( *it == n ) return TRUE;
		++it;
	}
	return FALSE;
}

void FenetrePrincipale::mettreAJourVecteurs()
{
	VisiteurVecteur v = VisiteurVecteur( this );

	vctPaquetage.erase( vctPaquetage.begin() , vctPaquetage.end() );
	vctClasse.erase( vctClasse.begin() , vctClasse.end() );
	vctRelation.erase( vctRelation.begin() , vctRelation.end() );

	Projet * p  = noyau->getProjet();
	DiagrammeStatique* dg = NULL ;
	if ( p != NULL ) dg = p->getDiagrammeStatique();
	if ( dg != NULL ) {
		dg->accepter(&v);
		noyau->notifier();
		cout << "MISE A JOUR des vecteurs russie : " << v.getNbAjout() << " lments" << endl;
	} else {
		cout << "MISE A JOUR des vecteurs impossible" << endl;
	}
}

int main(int argc, char **argv) {
	QApplication a( argc, argv );

	Noyau * n = new Noyau();
	FenetrePrincipale *f = new FenetrePrincipale(n);

	a.setStyle(new QPlatinumStyle());
	f->setGeometry(10,10,900,700);
	f->setCaption("UML v/t");
	a.setMainWidget( f );
	f->show();
	return a.exec();
}

