#ifndef FENETREPRINCIPALE_H
#define FENETREPRINCIPALE_H

#include <vector.h>

#include <qmainwindow.h>
#include <qtextbrowser.h>

#include <noynoyau.h>
#include <noynoeud.h>
#include <ihmbarreoutilsajouter.h>
#include <ihmbarreoutilsedition.h>
#include <ihmbarreoutilsprojet.h>
#include <ihmbarredemenu.h>
#include <ihmwidgetcentral.h>
#include <ihmfenetrepreference.h>

/** Cette classe represente la fenetre principale qui contiendra la barre de menu,
 * les barre d'outils et un widget central.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class FenetrePrincipale : public QMainWindow
{
	Q_OBJECT

	public:

		/** Constructeur
		 * \param Noyau *n : Represente le noyau associ au projet.
		 * \param QWidget *parent : Represente le widget parent de cet arbre (Par defaut 0).
		 * \param const char *name : Represente le nom que l'on veut donner  notre arbre (Par defaut 0).
		 */
		FenetrePrincipale(Noyau * n, QWidget *parent=0, const char *name=0);

		//! Destructeur
		virtual ~FenetrePrincipale() ;


		/** Permet de recuperer une reference sur le noyau qu'coute cette fenetre.
		 * \return Une reference sur le noyau qu'coute cette fenetre.
		 */
		Noyau * getNoyau() {return noyau;}

		//! Permet de positionner la variable prive noeudCourant.
		void setNoeudCourant( Noeud * n );

		/** Permet de recuperer une reference sur le noeud courant.
		 * \return Une reference sur le noeud courant.
		 */
		Noeud * getNoeudCourant() { return noeud; }


		/** Permet de recuperer une reference sur la barre de menu de cette fentre.
		 * \return Une reference sur la barre de menu de cette fentre.
		 */
		BarreDeMenu * getBarreDeMenu() {return barreDeMenu;}

		/** Permet de recuperer une reference sur la barre d'outils ajouter de cette fentre.
		 * \return Une reference sur la barre d'outils ajouter de cette fentre.
		 */
		BarreOutilsAjouter * getBarreOutilsAjouter() {return barreOutilsAjouter;}

		/** Permet de recuperer une reference sur la barre d'outils edition de cette fentre.
		 * \return Une reference sur la barre d'outils edition de cette fentre.
		 */
		BarreOutilsEdition * getBarreOutilsEdition() {return barreOutilsEdition;}

		/** Permet de recuperer une reference sur la barre d'outils projet de cette fentre.
		 * \return Une reference sur la barre d'outils projet de cette fentre.
		 */
		BarreOutilsProjet * getBarreOutilsProjet() {return barreOutilsProjet;}

		/** Permet de recuperer une reference sur le widget central de cette fentre.
		 * \return Une reference sur le widgt central de cette fentre.
		 */
		WidgetCentral * getWidgetCentral() {return widgetCentral;}


		std::vector<Paquetage*> vctPaquetage;
		
		/** Permet d'ajouter un element dans notre vecteur de paquetage vctPaquetage.
		 * \param Paquetage *p : Reference sur le paquetage  ajouter  vctPaquetage.
		 */
		void ajouterPaquetage( Paquetage * p );

		/** Permet de supprimer un element dans notre vecteur de paquetage vctPaquetage.
		 * \param Paquetage *p : Reference sur le paquetage  supprimer de vctPaquetage.
		 */
		void supprimerPaquetage( Paquetage * p );

		/** Permet de definir si un noeud est un paquetage ou non
		 * \param Noeud *n : Noeud  identifier comme paquetage ou non.
		 * \return TRUE si le parametre n est un paquetage, FALSE sinon.
		 */
		bool estPaquetage( Noeud * n );


		std::vector<Classe*> vctClasse;

		/** Permet d'ajouter un element dans notre vecteur de classe vctClasse.
		 * \param Classe *c : Reference sur la classe  ajouter  vctClasse.
		 */
		void ajouterClasse( Classe *c );

		/** Permet de supprimer un element de notre vecteur de classe vctClasse.
		 * \param Classe *c : Reference sur la classe  supprimer de vctClasse.
		 */
		void supprimerClasse( Classe *c );

		/** Permet de definir si un noeud est une classe ou non
		 * \param Noeud *n : Noeud  identifier comme classe ou non.
		 * \return TRUE si le parametre n est une classe, FALSE sinon.
		 */
		bool estClasse ( Noeud * n );

		std::vector<Relation*> vctRelation;

		/** Permet d'ajouter un element dans notre vecteur de relation vctRelation.
		 * \param Relation *r : Reference sur la relation  ajouter  vctRelation.
		 */
		void ajouterRelation( Relation *r );

		/** Permet de supprimer un element de notre vecteur de relation vctRelation.
		 * \param Relation *r : Reference sur la relation  supprimer de vctRelation.
		 */
		void supprimerRelation( Relation *r );

		/** Permet de definir si un noeud est une relation ou non
		 * \param Noeud *n : Noeud  identifier comme relation ou non.
		 * \return TRUE si le parametre n est une relation, FALSE sinon.
		 */
		bool estRelation ( Noeud * n );

		/** Permet de vider les vecteurs et de les remplir avec la nouvelle
		 * architecture, par exemple lors de l'import ou du chargement.
		 */
		void mettreAJourVecteurs();

		/** Positionne la variable prive fenetreDePreference.
		 * \param FenetreDePreference *fen : valeur  affecter  la variable prive fenetrePreference.
		 */
		void setFenetrePreference(FenetrePreference *fen) {
				fenetrePreference=fen;}

		/** Positionne la variable prive navigateur.
		 * \param QTextBrowser *fen : valeur  affecter  la variable prive navigateur.
		 */
		void setNavigateur(QTextBrowser *fen) {
				navigateur=fen;}

		/** Permet de recuperer une reference sur la fentre de preference associe  cette fentre.
		 * \return Une reference sur la fentre de preference associe  cette fentre.
		 */
		FenetrePreference * getFenetrePreference() {return fenetrePreference;}

		/** Permet de recuperer une reference sur le navigateur associ  cette fentre.
		 * \return Une reference sur le navigateur associ  cette fentre.
		 */
		QTextBrowser * getNavigateur() {return navigateur;}

	private:
		Noyau * noyau ;
		Noeud * noeud ;

		BarreDeMenu *barreDeMenu;
		BarreOutilsAjouter *barreOutilsAjouter;
		BarreOutilsEdition *barreOutilsEdition;
		BarreOutilsProjet *barreOutilsProjet;
		WidgetCentral *widgetCentral;
		FenetrePreference * fenetrePreference ;
		QTextBrowser *navigateur ;
};

#endif

