#include <qwidget.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qmessagebox.h>

#include <ihmfenetrerelation.h>

FenetreRelation::FenetreRelation(FenetrePrincipale * fen, Relation * r, int type_relation, QWidget *parent, const char *name)
	: QWidget(parent,name)
{
	fenetre = fen;
	relation = r;

	setCaption("Ajouter une relation");

	/* Classe source et destination */	 
	QGroupBox *groupSource = new QGroupBox (2, Horizontal,"Source", this, "groupSource");
	QGroupBox *groupDestination = new QGroupBox (2, Horizontal,"Destination", this, "groupDestination");
	if ( relation == NULL ) {
		if (fen->estClasse(fen->getNoeudCourant())) comboSource = new MonComboClasse ((Classe *)fen->getNoeudCourant(), fenetre, groupSource, "comboSource");
		else comboSource = new MonComboClasse (NULL, fenetre, groupSource, "comboSource");
		comboDestination = new MonComboClasse (NULL, fenetre, groupDestination, "comboDestination");
	} else {
		comboSource = new MonComboClasse (relation->getClasseSource(), fenetre, groupSource, "comboSource");
		comboDestination = new MonComboClasse ((Classe *)relation->getPere(), fenetre, groupDestination, "comboDestination");
	}
	editSource = new QLineEdit(groupSource, "editSource");
	editDestination = new QLineEdit(groupDestination, "editDesination");

	/* Zone de relation */
	QGroupBox *groupRelation = new QGroupBox (1, Vertical,"Type", this, "groupRelation");
	comboRelation = new QComboBox (groupRelation, "comboRelation");
	comboRelation->insertItem ( "Agrgation", 0 );
	comboRelation->insertItem ( "Utilisation", 1 );
	comboRelation->insertItem ( "Association", 2 );
	comboRelation->insertItem ( "Hritage", 3 );

	/* Description */
	QGroupBox *groupDescription = new QGroupBox (1, Vertical,"Description", this, "groupDescription" );
	editDescription = new QMultiLineEdit(groupDescription , "editDescription");

	/* Grille pour placer les lments de la fentre */
	QGridLayout *grille = new QGridLayout( this, 9, 1, 1 );
	QGridLayout *bouton = new QGridLayout( 1, 3, 1);
 
	/* placement des lments */
	grille->addWidget(groupSource, 1, 0);
	grille->addWidget(groupRelation, 3, 0);
	grille->addWidget(groupDestination, 5, 0);
	grille->addWidget(groupDescription, 7, 0);
	grille->addLayout(bouton, 9, 0);

	/* Connection des boutons */
	QPushButton *ok = new QPushButton ( "&Ok", this, "ok");
	connect( ok, SIGNAL(clicked()), this, SLOT(ok()));
	bouton->addWidget(ok, 0, 1);

	if ( relation == NULL ) {
		setGeometry(100,100,400,300);
		QPushButton *annuler = new QPushButton ( "&Annuler", this, "annuler");
		connect( annuler, SIGNAL(clicked()), this, SLOT(annuler()));
		bouton->addWidget(annuler, 0, 2);
	} else {
		if ( parentWidget() == NULL ) setMinimumSize( 400 , 300 );
		else setMinimumSize(parentWidget()->size());
		
		adjustSize();

		editSource->setText( QString(relation->getMultipliciteSource().c_str()) );
		editDestination->setText( QString(relation->getMultipliciteDestination().c_str()) );
		editDescription->setText( QString(relation->getDescription().c_str()) );

		comboSource->setEnabled(FALSE);
		comboDestination->setEnabled(FALSE);
		comboRelation->setCurrentItem ( type_relation-200 );
		comboRelation->setEnabled(FALSE);
	}
}

void FenetreRelation::ok()
{
	if ( relation == NULL ) {
		Classe * c1 , * c2 ;
		Relation * r ;
		string nom ;
		QString type;

		std::vector<Classe*> vctClasse = fenetre->vctClasse ;
		std::vector<Classe*>::iterator it = vctClasse.begin();
		int indiceClasseSource = comboSource->currentItem();
		int i = 0;
		while( i < indiceClasseSource ){
			++it;
			++i;
		}
		c1 = *it ;

		it = vctClasse.begin();
		int indiceClasseDestination = comboDestination->currentItem();
		i = 0;
		while( i < indiceClasseDestination ){
			++it;
			++i;
		}
		c2 = *it ;

		type = comboRelation->currentText();
		if ( type.compare("Agrgation") == 0 ) {
			nom = string(string(type) + " de " + c1->getNomComplet() + " vers " + c2->getNomComplet());

		} else {
			if ( type.compare("Utilisation") == 0 ) {
				nom = string(string(type) + " de " + c2->getNomComplet() + " par " + c1->getNomComplet());

			} else {
				if ( type.compare("Association") == 0 ) {
					nom = string(string(type) + " entre " + c1->getNomComplet() + " et " + c2->getNomComplet());

				} else { // "Hritage"
					if ( c1 == c2 ) {
						QMessageBox::warning(0,"Ajouter une relation","Une classe ne peut hriter d'elle mme");
						return;
					}
					nom = string(string(type) + " de la classe mre " + c2->getNomComplet());

				}
			}
		}
		r = c1->ajouterRelation( nom, comboRelation->currentItem()+200, 
						c1, c2, string(editSource->text()), string(editDestination->text()) );

		if ( r != NULL ) {
				r->setDescription( string(editDescription->text()) );
				fenetre->ajouterRelation(r);
				cout << "AJOUT (relation) " << r << endl;
				std::vector<Vue*> v = fenetre->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
				std::vector<Vue*>::iterator itv;
				Noeud * no = fenetre->getNoeudCourant();
				itv = v.begin();
				while (itv != v.end()){
					if ((*itv) == no){
						(*itv)->ajouterElementDeModelisation((ElementDeModelisation*)r, 20 , 20 , 20 , 20 , true);
						break;
					}
					++itv;
				}
				delete(this);
		} else
				QMessageBox::warning(0,"Ajouter une relation",
				"Impossible d'ajouter une relation avec ces renseignements\nVrifier qu'une relation identique n'existe pas");


	} else {
		relation->setMultipliciteSource( string(editSource->text()) );
		relation->setMultipliciteDestination( string(editDestination->text()) );
		relation->setDescription( string(editDescription->text()) );
		if ( parentWidget() == NULL ) delete(this);
	}
	fenetre->getNoyau()->notifier();
}

void FenetreRelation::annuler()
{
	delete(this);
}

