#ifndef IHMVISITEURAJOUTER_H
#define IHMVISITEURAJOUTER_H

#include <qlistview.h>

#include <noyvisiteurnoeud.h>
#include <noyprojet.h>
#include <noynoyau.h>

class FenetrePrincipale;

/** Cette classe represente le visiteur associ  un ajout.
 * Elle demande les informations necessaire au noyau pour ajouter un lment de l'arbre.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class VisiteurAjouter : public VisiteurNoeud
{
	public:

		//! Constructeur
		VisiteurAjouter(FenetrePrincipale * fen);

		//! Destructeur
		virtual ~VisiteurAjouter() {}


		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Projet.
		 * \param Projet *p : Reference sur le projet  traiter.
		 */
		void traiterProjet(Projet *p);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type UseCase.
		 * \param UseCase *u : Reference sur le usecase  traiter.
		 */
		void traiterUseCase(UseCase *u);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type collaboration.
		 * \param Collaboration *c : Reference sur le diagramme de collaboration  traiter.
		 */
		void traiterCollaboration(Collaboration *c);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type DiagrammeStatique.
		 * \param DiagrammeStatique *d : Reference sur le diagramme statique  traiter.
		 */
		void traiterDiagrammeStatique(DiagrammeStatique *d);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Vue.
		 * \param Vue *v : Reference sur la vue  traiter.
		 */
		void traiterVue(Vue *v);

		/** Cette mthode s'occupe de l'ajout d'un objet affichable d'une vue.
		 * \param ObjetAffichable *o : Reference sur la vue  traiter.
		 */
		void traiterObjetAffichable(ObjetAffichable *o);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Classe.
		 * \param Classe *c : Reference sur la classe  traiter.
		 */
		void traiterClasse(Classe *cl);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Paquetage.
		 * \param Paquetage *p : Reference sur le paquetage  traiter.
		 */
		void traiterPaquetage(Paquetage *pa);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Methode.
		 * \param Methode *m : Reference sur la mthode  traiter.
		 */
		void traiterMethode(Methode *m);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Attribut.
		 * \param Attribut *a : Reference sur l'attribut  traiter.
		 */
		void traiterAttribut(Attribut *attr);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Heritage.
		 * \param Heritage *h : Reference sur l'heritage  traiter.
		 */
		void traiterHeritage(Heritage *h );

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Utilisation.
		 * \param Utilisation *u : Reference sur l'utilisation  traiter.
		 */
		void traiterUtilisation(Utilisation *u);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Association.
		 * \param Association *a : Reference sur l'association  traiter.
		 */
		void traiterAssociation(Association *assoc);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Agregation.
		 * \param Agregation *a : Reference sur l'agregation  traiter.
		 */
		void traiterAgregation(Agregation *ag);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type RelationCommentaire.
		 * \param RelationCommentaire *rc : Reference sur la relation commentaire  traiter.
		 */
		void traiterRelationCommentaire(RelationCommentaire *rc);

	private:
		FenetrePrincipale * fenetre;
};

#endif

