#include <qstring.h>
#include <qpixmap.h>

#include <ihmarbre.h>
#include <ihmfenetreprincipale.h>
#include <noyrelationcommentaire.h>

VisiteurArbre::VisiteurArbre(Noyau *n, Arbre *arbre)
{
	p = n->getProjet();
	a = arbre;
}

void VisiteurArbre::traiterProjet(Projet *p)
{
	QString *nomQt = new QString(p->getNom().c_str());
	MonListViewItem *projet = new MonListViewItem((Noeud *)p, a, *nomQt);
	a->setItemCourant(projet);
	(p->getDiagrammeStatique())->accepter(this);
	a->setItemCourant(projet);
	(p->getCollaboration())->accepter(this);
	a->setItemCourant(projet);
	(p->getUseCase())->accepter(this);
	a->setItemCourant(projet);
	a->setOpen(projet, TRUE);

	if (p == (Projet *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = projet;
}

void VisiteurArbre::traiterUseCase(UseCase *u)
{
	QString *nomQt = new QString(u->getNom().c_str());
	MonListViewItem *utilisation = new MonListViewItem((Noeud *)u, a->getItemCourant(), *nomQt);
	a->setOpen(utilisation, TRUE);
	a->setItemCourant(utilisation);

	if (u == (UseCase *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = utilisation;
}

void VisiteurArbre::traiterCollaboration(Collaboration *c)
{
	QString *nomQt = new QString(c->getNom().c_str());
	MonListViewItem *collaboration = new MonListViewItem((Noeud *)c, a->getItemCourant(), *nomQt);
	a->setItemCourant(collaboration);
	a->setOpen(collaboration, TRUE);

	if (c == (Collaboration *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = collaboration;
}

void VisiteurArbre::traiterDiagrammeStatique(DiagrammeStatique *d)
{
	QString *nomQt = new QString(d->getNom().c_str());
	MonListViewItem *statique = new MonListViewItem((Noeud *)d, a->getItemCourant(), *nomQt);
	a->setItemCourant(statique);

	std::vector<Vue *> vecVue;
	vecVue = d->getVctVue();
	std::vector<Vue *>::iterator iterateur1 = vecVue.begin();
	std::vector<Objetuml *> vecObjetuml;
	vecObjetuml = d->getVctObjetuml();
	std::vector<Objetuml *>::iterator iterateur2 = vecObjetuml.begin();

	// On cre le listViewItem Vue
	MonListViewItem *vues = new MonListViewItem((Noeud *)d, a->getItemCourant(), "Diagrammes");
	MonListViewItem *objetsUml = new MonListViewItem((Noeud *)d, a->getItemCourant(), "Objets Uml");

	while (iterateur1 != vecVue.end())
	{
		a->setItemCourant(vues);
		(*iterateur1)->accepter(this);
		++iterateur1;
	}

	while (iterateur2 != vecObjetuml.end())
	{
		a->setItemCourant(objetsUml);
		(*iterateur2)->accepter(this);
		++iterateur2;
	}

	a->setOpen(statique, TRUE);
	a->setOpen(vues, TRUE);
	a->setOpen(objetsUml, TRUE);

	if (d == (DiagrammeStatique *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = statique;
}

void VisiteurArbre::traiterVue(Vue *v)
{
	QString *nomQt = new QString(v->getNom().c_str());
	MonListViewItem *vue = new MonListViewItem((Noeud *)v, a->getItemCourant(), *nomQt);
	a->setItemCourant(vue);

	MonListViewItem *buffer;
	QString *buffer_nomQt;

	std::vector<ObjetAffichable *> vec;
	vec=v->getvctObjetAffichable();
	std::vector<ObjetAffichable *>::iterator iterateur = vec.begin();

	while (iterateur != vec.end())
	{
		buffer_nomQt = new QString((*iterateur)->getedm()->getNom().c_str());
		if (((ObjetAffichable *)(*iterateur))->getedm()->getPere() == v) {
			buffer = new MonListViewItem(((ObjetAffichable *)(*iterateur))->getedm(), vue, *buffer_nomQt);
			a->setItemCourant(buffer);
			((Commentaire*)((*iterateur)->getedm()))->accepter(this);
			a->setOpen(buffer,TRUE);
			a->setItemCourant(vue);
		}
		else
			buffer = new MonListViewItem((Noeud *)(*iterateur), vue, *buffer_nomQt);

		if ((*iterateur) == (ObjetAffichable *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
			a->itemASelectionner = buffer;

		++iterateur;
	}

	a->setOpen(vue, TRUE);

	if (v == (Vue *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = vue;
}

void VisiteurArbre::traiterCommentaire(Commentaire *c)
{
	QString *buffer_nomQt;
	MonListViewItem *buffer;

	std::vector<RelationCommentaire *> vct = c->getvctRelCom();
	std::vector<RelationCommentaire *>::iterator it = vct.begin();

	while ( it != vct.end() ) {
		buffer_nomQt = new QString( (*it)->getNom().c_str() );
		buffer = new MonListViewItem( *it , a->getItemCourant() , *buffer_nomQt);
		++it;
	}
}

void VisiteurArbre::traiterClasse(Classe *cl)
{
	QString *nomQt = new QString(cl->getNom().c_str());

	MonListViewItem *classe = new MonListViewItem((Noeud *)cl, a->getItemCourant(), *nomQt);
	classe->setPixmap(0, QPixmap("xpms/Classe.xpm"));

	std::vector<Methode *> vecMethode;
	vecMethode = cl->getVctMethode();
	std::vector<Methode *>::iterator iterateur1 = vecMethode.begin();

	std::vector<Attribut *> vecAttribut;
	vecAttribut = cl->getVctAttribut();
	std::vector<Attribut *>::iterator iterateur2 = vecAttribut.begin();

	std::vector<Relation *> vecRelation;
	vecRelation = cl->getVctRelation();
	std::vector<Relation *>::iterator iterateur3 = vecRelation.begin();

	while (iterateur1 != vecMethode.end())
	{
		a->setItemCourant(classe);
		(*iterateur1)->accepter(this);
		++iterateur1;
	}

	while (iterateur2 != vecAttribut.end())
	{
		a->setItemCourant(classe);
		(*iterateur2)->accepter(this);
		++iterateur2;
	}

	while (iterateur3 != vecRelation.end())
	{
		a->setItemCourant(classe);
		if ( (*iterateur3)->getClasseSource() == cl ) (*iterateur3)->accepter(this);
		++iterateur3;
	}

	a->setOpen(classe, TRUE);

	if (cl == (Classe *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = classe;
}

void VisiteurArbre::traiterPaquetage(Paquetage *pa)
{
	QString *nomQt = new QString(pa->getNom().c_str());

	MonListViewItem *paquetage = new MonListViewItem((Noeud *)pa, a->getItemCourant(), *nomQt);
	paquetage->setPixmap(0, QPixmap("xpms/Paquetage.xpm"));

	std::vector<Objetuml *> vecObjetUml;
	vecObjetUml = pa->getVctObjetuml();
	std::vector<Objetuml *>::iterator iterateur = vecObjetUml.begin();

	while (iterateur != vecObjetUml.end())
	{
		a->setItemCourant(paquetage);
		(*iterateur)->accepter(this);
		++iterateur;
	}

	a->setOpen(paquetage,TRUE);

	if (pa == (Paquetage *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = paquetage;
}

void VisiteurArbre::traiterObjetuml(Objetuml *o)
{
}

void VisiteurArbre::traiterMethode(Methode *m)
{
	QString *nomQt = new QString(m->getNom().c_str());
	MonListViewItem *methode;
	methode = new MonListViewItem((Noeud *)m, a->getItemCourant(), *nomQt);
	methode->setPixmap(0, QPixmap("xpms/Methode.xpm"));

	if (m == (Methode *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = methode;
}

void VisiteurArbre::traiterRelation(Relation *r)
{
/*	QString *nomQt = new QString(r->getNom().c_str());
	MonListViewItem *relation;
	relation = new MonListViewItem((Noeud *)r, a->getItemCourant(), *nomQt);

	if (r == (Relation *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = relation;
*/}

void VisiteurArbre::traiterAttribut(Attribut *attr)
{
	QString *nomQt = new QString(attr->getNom().c_str());
	MonListViewItem *attribut;
	attribut = new MonListViewItem((Noeud *)attr, a->getItemCourant(), *nomQt);
	attribut->setPixmap(0, QPixmap("xpms/Attribut.xpm"));

	if (attr == (Attribut *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = attribut;
}

void VisiteurArbre::traiterHeritage(Heritage *h)
{
	QString *nomQt = new QString(h->getNom().c_str());
	MonListViewItem *heritage;
	heritage = new MonListViewItem((Noeud *)h, a->getItemCourant(), *nomQt);
	heritage->setPixmap(0, QPixmap("xpms/Relation.xpm"));

	if (h == (Heritage *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = heritage;
}

void VisiteurArbre::traiterUtilisation(Utilisation *u)
{
	QString *nomQt = new QString(u->getNom().c_str());
	MonListViewItem *utilisation;
	utilisation = new MonListViewItem((Noeud *)u, a->getItemCourant(), *nomQt);
	utilisation->setPixmap(0, QPixmap("xpms/Relation.xpm"));

	if (u == (Utilisation *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = utilisation;
}

void VisiteurArbre::traiterAssociation(Association *assoc)
{
	QString *nomQt = new QString(assoc->getNom().c_str());
	MonListViewItem *association;
	association = new MonListViewItem((Noeud *)assoc, a->getItemCourant(), *nomQt);
	association->setPixmap(0, QPixmap("xpms/Relation.xpm"));

	if (assoc == (Association *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = association;
}

void VisiteurArbre::traiterAgregation(Agregation *ag)
{
	QString *nomQt = new QString(ag->getNom().c_str());
	MonListViewItem *agregation;
	agregation = new MonListViewItem((Noeud *)ag, a->getItemCourant(), *nomQt);
	agregation->setPixmap(0, QPixmap("xpms/Relation.xpm"));

	if (ag == (Agregation *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = agregation;
}

void VisiteurArbre::traiterRelationCommentaire(RelationCommentaire *rc)
{
	QString *nomQt = new QString(rc->getNom().c_str());
	MonListViewItem *relationCommentaire;
	relationCommentaire = new MonListViewItem((Noeud *)rc, a->getItemCourant(), *nomQt);

	if (rc == (RelationCommentaire *)((FenetrePrincipale *)(a->topLevelWidget()))->getNoeudCourant())
		a->itemASelectionner = relationCommentaire;
}
