#ifndef IHMVISITEURARBRE_H
#define IHMVISITEURARBRE_H

#include <qlistview.h>

#include <noyvisiteurnoeud.h>
#include <noyprojet.h>
#include <noynoyau.h>

class Arbre;

/** Cette classe represente le visiteur associ  un arbre.
 * Elle demande les informations necessaire au noyau puis dessine l'arbre.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class VisiteurArbre : public VisiteurNoeud
{
	private:
		Projet *p;
		Arbre *a;

	public:

		/** Constructeur
		 * \param Noyau *n : Represente le noyau associ  l'application.
		 * \param Arbre *arbre : Reference sur l'arbre  dessiner.
		 */
		VisiteurArbre(Noyau *n, Arbre *arbre);

		//! Destructeur
		virtual ~VisiteurArbre() {}


		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Projet dans l'arbre.
		 * \param Projet *p : Reference sur le projet  traiter.
		 */
		void traiterProjet(Projet *p);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type UseCase dans l'arbre.
		 * \param UseCase *u : Reference sur le usecase  traiter.
		 */
		void traiterUseCase(UseCase *u);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type collaboration dans l'arbre.
		 * \param Collaboration *c : Reference sur le diagramme de collaboration  traiter.
		 */
		void traiterCollaboration(Collaboration *c);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type DiagrammeStatique dans l'arbre.
		 * \param DiagrammeStatique *d : Reference sur le diagramme statique  traiter.
		 */
		void traiterDiagrammeStatique(DiagrammeStatique *d);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Vue dans l'arbre.
		 * \param Vue *v : Reference sur la vue  traiter.
		 */
		void traiterVue(Vue *v);

		/** Cette mthode s'occupe de l'ajout des Noeuds fils d'un commentaire
		 * \param Commentaire *c : Reference sur le commentaire  traiter.
		 */
		void traiterCommentaire(Commentaire *c);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Classe dans l'arbre.
		 * \param Classe *c : Reference sur la classe  traiter.
		 */
		void traiterClasse(Classe *cl);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Paquetage dans l'arbre.
		 * \param Paquetage *p : Reference sur le paquetage  traiter.
		 */
		void traiterPaquetage(Paquetage *pa);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Objetuml dans l'arbre (i.e. elle ne fait rien).
		 * \param Objetuml *o : Reference sur l'objet UML  traiter.
		 */
		void traiterObjetuml(Objetuml *o);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Methode dans l'arbre.
		 * \param Methode *m : Reference sur la mthode  traiter.
		 */
		void traiterMethode(Methode *m);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Relation dans l'arbre.
		 * \param Relation *r : Reference sur la relation  traiter.
		 */
		void traiterRelation(Relation *r);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Attribut dans l'arbre.
		 * \param Attribut *a : Reference sur l'attribut  traiter.
		 */
		void traiterAttribut(Attribut *attr);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Heritage dans l'arbre.
		 * \param Heritage *h : Reference sur l'heritage  traiter.
		 */
		void traiterHeritage(Heritage *h );

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Utilisation dans l'arbre.
		 * \param Utilisation *u : Reference sur l'utilisation  traiter.
		 */
		void traiterUtilisation(Utilisation *u);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Association dans l'arbre.
		 * \param Association *a : Reference sur l'association  traiter.
		 */
		void traiterAssociation(Association *assoc);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type Agregation dans l'arbre.
		 * \param Agregation *a : Reference sur l'agregation  traiter.
		 */
		void traiterAgregation(Agregation *ag);

		/** Cette mthode s'occupe de l'ajout d'un Noeud de type RelationCommentaire dans l'arbre;
		 * \param RelationCommentaire *rc : Reference sur la relation commentaire  traiter;
		 */
		void traiterRelationCommentaire(RelationCommentaire *rc);
};

#endif

