#include <qkeycode.h>
#include <qaccel.h>
#include <qevent.h>
#include <qscrollview.h>

#include <ihmvisiteuritem.h>
#include <ihmzonedessin.h>
#include <noyrelationcommentaire.h>

VisiteurItem::VisiteurItem(FenetrePrincipale *fenetre)
	: VisiteurNoeud()
{
	curseur = new QCursor(PointingHandCursor);

	f=fenetre;
}

void VisiteurItem::traiterProjet(Projet *p)
{
	QPopupMenu *projet = new QPopupMenu(f);
	QPopupMenu *exporter = new QPopupMenu(projet);
	QPopupMenu *parser = new QPopupMenu(projet);

	CHECK_PTR(exporter);
	exporter->insertItem("C++", f->getBarreDeMenu(), SLOT(exportCpp())); 
	exporter->insertItem("Java", f->getBarreDeMenu(), SLOT(exportJava()));	

	CHECK_PTR(parser);
	parser->insertItem("C++", f->getBarreDeMenu(), SLOT(importCpp()));
	parser->insertItem("Java", f->getBarreDeMenu(), SLOT(importJava()));

	CHECK_PTR(projet);
	projet->insertItem("Enregi&strer", f->getBarreDeMenu(), SLOT(enregistrer()), QAccel::stringToKey(QObject::tr("Ctrl+S")));
	projet->insertItem("&Informations", f->getBarreDeMenu(), SLOT(informations()), QAccel::stringToKey(QObject::tr("Ctrl+I")));
	projet->insertSeparator();
	projet->insertItem("Exporter", exporter);
	projet->insertItem("Parser", parser);

	projet->setCursor(*curseur);

	projet->setGeometry(curseur->pos().x(), curseur->pos().y(), projet->width(), projet->height());

	projet->show();
}

void VisiteurItem::traiterUseCase(UseCase *u)
{
	QPopupMenu *useCase = new QPopupMenu(f);

	CHECK_PTR(useCase);
	useCase->insertItem("Noeud sans correspondance");

	useCase->setCursor(*curseur);

	useCase->setGeometry(curseur->pos().x(), curseur->pos().y(), useCase->width(), useCase->height());

	useCase->show();
}

void VisiteurItem::traiterCollaboration(Collaboration *c)
{
	QPopupMenu *collaboration = new QPopupMenu(f);

	CHECK_PTR(collaboration);
	collaboration->insertItem("Noeud sans correspondance");

	collaboration->setCursor(*curseur);

	collaboration->setGeometry(curseur->pos().x(), curseur->pos().y(), collaboration->width(), collaboration->height());

	collaboration->show();
}

void VisiteurItem::traiterDiagrammeStatique(DiagrammeStatique *d)
{
	QPopupMenu *collaboration = new QPopupMenu(f);

	CHECK_PTR(collaboration);
	collaboration->insertItem("Noeud sans correspondance");

	collaboration->setCursor(*curseur);

	collaboration->setGeometry(curseur->pos().x(), curseur->pos().y(), collaboration->width(), collaboration->height());

	collaboration->show();

}

void VisiteurItem::traiterVue(Vue *v)
{
	QPopupMenu *vue = new QPopupMenu(f);
	ZoneDessin *zone = (ZoneDessin *)((QScrollView *)f->getWidgetCentral()->getFenetreDeDessin()->getConteneur())->child("zoneDessin");
	CHECK_PTR(vue);
	vue->insertItem("Creer un paquetage", f->getBarreDeMenu(), SLOT(paquetage()));
	vue->insertItem("Creer une classe",f->getBarreDeMenu(), SLOT(classe()));
	vue->insertItem("Creer une relation", f->getBarreDeMenu(), SLOT(relation()));
	vue->insertSeparator();
	vue->insertItem("Ajouter un paquetage", f->getWidgetCentral()->getArbre(), SLOT(paquetageVersVue()));
	vue->insertItem("Ajouter un commentaire", f->getBarreDeMenu(), SLOT(commentaire()));
	vue->insertItem("Ajouter une classe", f->getWidgetCentral()->getArbre(), SLOT(classeVersVue()));
	vue->insertItem("Ajouter une relation", f->getWidgetCentral()->getArbre(), SLOT(relationVersVue()));
	vue->insertSeparator();
	vue->insertItem("Enregistrer la vue", zone, SLOT(enregistrerVue()));
	vue->insertSeparator();
	vue->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	vue->setCursor(*curseur);

	vue->setGeometry(curseur->pos().x(), curseur->pos().y(), vue->width(), vue->height());

	vue->show();
}

void VisiteurItem::traiterObjetAffichable(ObjetAffichable *oa)
{
	QPopupMenu *objetaff = new QPopupMenu(f);

	CHECK_PTR(objetaff);
	objetaff->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	objetaff->setCursor(*curseur);

	objetaff->setGeometry(curseur->pos().x(), curseur->pos().y(), objetaff->width(), objetaff->height());

	objetaff->show();
}

void VisiteurItem::traiterCommentaire(Commentaire *c)
{
	QPopupMenu *commentaire = new QPopupMenu(f);
	QPopupMenu *objets = new QPopupMenu(commentaire);

	int itemID = -1;

	CHECK_PTR(objets);
	std::vector<ObjetAffichable *> vecObj = ((Vue *)c->getPere())->getvctObjetAffichable();
	std::vector<ObjetAffichable *>::iterator ito = vecObj.begin();
	ElementDeModelisation * edm ;
	while ( ito != vecObj.end() ) {
		edm = (*ito)->getedm() ;

		if ( edm != c) {
			objets->insertItem( QString(edm->getNom().c_str()), ++itemID);
			if ( f->estClasse( edm ) ) {
					std::vector<Methode *> vctM = ((Classe*)(edm))->getVctMethode();
					std::vector<Methode *>::iterator itM = vctM.begin() ;
					while ( itM != vctM.end() ) {
						objets->insertItem( QString(  string("|--"+(*itM)->getNom()).c_str() ), ++itemID);
						++itM;
					}

					std::vector<Attribut *> vctA = ((Classe*)(edm))->getVctAttribut();
					std::vector<Attribut *>::iterator itA = vctA.begin() ;
					while ( itA != vctA.end() ) {
						objets->insertItem( QString(  string("|--"+(*itA)->getNom()).c_str() ), ++itemID);
						++itA;
					}
			}
		}
		++ito;
	}
	QObject::connect( objets , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(lierCommentaire(int)) );

	CHECK_PTR(commentaire);
	commentaire->insertItem("Lier  ...", objets);
	commentaire->insertSeparator();
	commentaire->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	commentaire->setCursor(*curseur);

	commentaire->setGeometry(curseur->pos().x(), curseur->pos().y(), commentaire->width(), commentaire->height());

	commentaire->show();
}

void VisiteurItem::traiterRelationCommentaire(RelationCommentaire *rc)
{
	QPopupMenu *relationCommentaire = new QPopupMenu(f);
	
	CHECK_PTR(relationCommentaire);
	relationCommentaire->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));
	relationCommentaire->setCursor(*curseur);
	relationCommentaire->setGeometry(curseur->pos().x(), curseur->pos().y(), relationCommentaire->width(), relationCommentaire->height());
	relationCommentaire->show();
}

void VisiteurItem::traiterClasse(Classe *cl)
{
	QPopupMenu *classe = new QPopupMenu(f);
	QPopupMenu *vues = new QPopupMenu(classe);

	int itemID = -1;
	
	CHECK_PTR(vues);
	std::vector<Vue*>::iterator it;
	std::vector<Vue*> vecVues;
	vecVues = f->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
	it = vecVues.begin();
	while ( it != vecVues.end() ) {
		vues->insertItem( QString( (*it)->getNom().c_str() ) , ++itemID );
		++it;
	}
	QObject::connect( vues , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(ajouterDansVue(int)) );

	CHECK_PTR(classe);
	classe->insertItem("Ajouter dans une vue ...", vues);
	classe->insertSeparator();
	classe->insertItem("Ajouter une methode", f->getBarreDeMenu(), SLOT(methode()));
	classe->insertItem("Ajouter un attribut", f->getBarreDeMenu(), SLOT(attribut()));
	classe->insertItem("Ajouter une relation", f->getBarreDeMenu(), SLOT(relation()));
	classe->insertSeparator();
	classe->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	classe->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	classe->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	classe->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	classe->setCursor(*curseur);

	classe->setGeometry(curseur->pos().x(), curseur->pos().y(), classe->width(), classe->height());

	classe->show();
}

void VisiteurItem::traiterPaquetage(Paquetage *pa)
{
	QPopupMenu *paquetage = new QPopupMenu(f);
	QPopupMenu *vues = new QPopupMenu(paquetage);

	int itemID = -1 ;

	CHECK_PTR(vues);
	std::vector<Vue*>::iterator it;
	std::vector<Vue*> vecVues;
	vecVues = f->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
	it = vecVues.begin();
	while ( it != vecVues.end() ) {
		vues->insertItem( QString((*it)->getNom().c_str()) , ++itemID );
		++it;
	}
	QObject::connect( vues , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(ajouterDansVue(int)) );

	CHECK_PTR(paquetage);
	paquetage->insertItem("Ajouter dans une vue ...", vues);
	paquetage->insertSeparator();
	paquetage->insertItem("Ajouter un paquetage", f->getBarreDeMenu(), SLOT(paquetage()));
	paquetage->insertItem("Ajouter une classe", f->getBarreDeMenu(), SLOT(classe()));
	paquetage->insertSeparator();
	paquetage->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	paquetage->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	paquetage->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	paquetage->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));
	
	paquetage->setCursor(*curseur);

	paquetage->setGeometry(curseur->pos().x(), curseur->pos().y(), paquetage->width(), paquetage->height());

	paquetage->show();
}

/*
void VisiteurItem::traiterObjetuml(Objetuml *o)
{
	QPopupMenu *objetUml = new QPopupMenu(f);

	CHECK_PTR(objetUml);
	objetUml->insertItem("Ajouter un paquetage", f->getBarreDeMenu(), SLOT(paquetage()));
	objetUml->insertItem("Ajouter une classe", f->getBarreDeMenu(), SLOT(classe()));

	objetUml->setCursor(*curseur);

	objetUml->setGeometry(curseur->pos().x(), curseur->pos().y(), objetUml->width(), objetUml->height());

	objetUml->show();
}
*/

void VisiteurItem::traiterMethode(Methode *m)
{
	QPopupMenu *methode = new QPopupMenu(f);

	CHECK_PTR(methode);
	if (((Classe*)(m->getPere()))->getTypeDeClasse() == true){
		methode->insertItem("Editer code", f->getWidgetCentral()->getArbre(), SLOT(editionCode()));
		methode->insertSeparator();
	}
	methode->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	methode->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	methode->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	methode->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	methode->setCursor(*curseur);

	methode->setGeometry(curseur->pos().x(), curseur->pos().y(), methode->width(), methode->height());
	
	methode->show();
}

/*
void VisiteurItem::traiterRelation(Relation *r)
{
	QPopupMenu *relation = new QPopupMenu(f);

	CHECK_PTR(relation);
	relation->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	relation->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	relation->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	relation->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	relation->setCursor(*curseur);

	relation->setGeometry(curseur->pos().x(), curseur->pos().y(), relation->width(), relation->height());

	relation->show();
}
*/

void VisiteurItem::traiterAttribut(Attribut *a)
{
	QPopupMenu *attribut = new QPopupMenu(f);

	CHECK_PTR(attribut);
	attribut->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	attribut->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	attribut->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	attribut->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	attribut->setCursor(*curseur);

	attribut->setGeometry(curseur->pos().x(), curseur->pos().y(), attribut->width(), attribut->height());

	attribut->show();
}

void VisiteurItem::traiterHeritage(Heritage *h)
{
	QPopupMenu *heritage = new QPopupMenu(f);
	QPopupMenu *vues = new QPopupMenu(heritage);

	int itemID = -1;
	
	CHECK_PTR(vues);
	std::vector<Vue*>::iterator it;
	std::vector<Vue*> vecVues;
	vecVues = f->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
	it = vecVues.begin();
	while ( it != vecVues.end() ) {
		vues->insertItem( QString( (*it)->getNom().c_str() ) , ++itemID );
		++it;
	}
	QObject::connect( vues , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(ajouterDansVue(int)) );

	CHECK_PTR(heritage);
	heritage->insertItem("Ajouter dans une vue...", vues);
	heritage->insertSeparator();
	heritage->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	heritage->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	heritage->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	heritage->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	heritage->setCursor(*curseur);

	heritage->setGeometry(curseur->pos().x(), curseur->pos().y(), heritage->width(), heritage->height());

	heritage->show();
}

void VisiteurItem::traiterAssociation(Association *assoc)
{
	QPopupMenu *association = new QPopupMenu(f);
	QPopupMenu *vues = new QPopupMenu(association);

	int itemID = -1;
	
	CHECK_PTR(vues);
	std::vector<Vue*>::iterator it;
	std::vector<Vue*> vecVues;
	vecVues = f->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
	it = vecVues.begin();
	while ( it != vecVues.end() ) {
		vues->insertItem( QString( (*it)->getNom().c_str() ) , ++itemID );
		++it;
	}
	QObject::connect( vues , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(ajouterDansVue(int)) );


	CHECK_PTR(association);
	association->insertItem("Ajouter dans une vue...", vues);
	association->insertSeparator();
	association->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	association->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	association->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	association->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	association->setCursor(*curseur);

	association->setGeometry(curseur->pos().x(), curseur->pos().y(), association->width(), association->height());

	association->show();
}

void VisiteurItem::traiterUtilisation(Utilisation *u)
{
	QPopupMenu *utilisation = new QPopupMenu(f);
	QPopupMenu *vues = new QPopupMenu(utilisation);

	int itemID = -1;
	
	CHECK_PTR(vues);
	std::vector<Vue*>::iterator it;
	std::vector<Vue*> vecVues;
	vecVues = f->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
	it = vecVues.begin();
	while ( it != vecVues.end() ) {
		vues->insertItem( QString( (*it)->getNom().c_str() ) , ++itemID );
		++it;
	}
	QObject::connect( vues , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(ajouterDansVue(int)) );


	CHECK_PTR(utilisation);
	utilisation->insertItem("Ajouter dans une vue...", vues);
	utilisation->insertSeparator();
	utilisation->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	utilisation->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	utilisation->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	utilisation->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	utilisation->setCursor(*curseur);

	utilisation->setGeometry(curseur->pos().x(), curseur->pos().y(), utilisation->width(), utilisation->height());

	utilisation->show();
}

void VisiteurItem::traiterAgregation(Agregation *a)
{
	QPopupMenu *agregation = new QPopupMenu(f);
	QPopupMenu *vues = new QPopupMenu(agregation);

	int itemID = -1;
	
	CHECK_PTR(vues);
	std::vector<Vue*>::iterator it;
	std::vector<Vue*> vecVues;
	vecVues = f->getNoyau()->getProjet()->getDiagrammeStatique()->getVctVue();
	it = vecVues.begin();
	while ( it != vecVues.end() ) {
		vues->insertItem( QString( (*it)->getNom().c_str() ) , ++itemID );
		++it;
	}
	QObject::connect( vues , SIGNAL(activated(int )) , f->getBarreDeMenu() , SLOT(ajouterDansVue(int)) );


	CHECK_PTR(agregation);
	agregation->insertItem("Ajouter dans une vue...", vues);
	agregation->insertSeparator();
	agregation->insertItem("Couper", f->getBarreDeMenu(), SLOT(couper()));
	agregation->insertItem("Copier", f->getBarreDeMenu(), SLOT(copier()));
	agregation->insertItem("Coller", f->getBarreDeMenu(), SLOT(coller()));
	agregation->insertItem("Supprimer", f->getBarreDeMenu(), SLOT(supprimer()));

	agregation->setCursor(*curseur);

	agregation->setGeometry(curseur->pos().x(), curseur->pos().y(), agregation->width(), agregation->height());

	agregation->show();
}

