#include <vector.h>

#include <qstring.h>
#include <qpixmap.h>
#include <qmessagebox.h>

#include <ihmfenetreprincipale.h>
#include <ihmvisiteursupprimer.h>
#include <noyrelationcommentaire.h>


VisiteurSupprimer::VisiteurSupprimer(FenetrePrincipale * fen)
{
	fenetre = fen ;
}

void VisiteurSupprimer::traiterVue(Vue *v)
{
	DiagrammeStatique * dg = (DiagrammeStatique *)v->getPere();
	string s = v->getNom();

	std::vector<ObjetAffichable*> vct = v->getvctObjetAffichable();
	std::vector<ObjetAffichable*>::iterator it = vct.begin();

	while ( it != vct.end() ){ 
		(*it)->accepter(this);
		++it;
	}

	if ( dg->supprimerVue(v) == FALSE )
		QMessageBox::warning(0,"Erreur de suppression","Impossible de supprimer cette vue");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == v) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		delete(v);
		cout << "SUPPRESSION (vue) " << s << endl;
	}
}

void VisiteurSupprimer::traiterObjetAffichable(ObjetAffichable *o)
{
	Vue * v = (Vue *)o->getPere();
	string s = o->getedm()->getNom();

	if ( v->supprimerObjetAffichable(o) == FALSE ){
		QMessageBox::warning(0,"Erreur de suppression","Impossible de supprimer cet objet");
	}
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == o) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (ObjetAffichable) associ  " << s << endl;
	}
}

void VisiteurSupprimer::traiterClasse(Classe *cl)
{
	Noeud * noeud_pere = cl->getPere();
	DiagrammeStatique * dg = fenetre->getNoyau()->getProjet()->getDiagrammeStatique();
	string s = cl->getNomComplet();

	fenetre->supprimerClasse( (Classe *)cl);
	cout << "SUPPRESSION (classe) " << s << endl;

	if ( noeud_pere == dg ) {
		if ( dg->supprimerObjetuml( (Objetuml *)cl ) == FALSE )
				QMessageBox::warning(0, "Erreur de suppression", "Impossible de supprimer cette classe dans le diagramme statique");
		else if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == cl) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
	} else {
		if ( ((Paquetage *)noeud_pere)->supprimerObjetuml( (Objetuml *)cl ) == FALSE )
				QMessageBox::warning(0, "Erreur de suppression", "Impossible de supprimer cette classe dans ce paquetage");
		else if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == cl) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
	}
}

void VisiteurSupprimer::traiterPaquetage(Paquetage *pa)
{
	Noeud * noeud_pere = pa->getPere();
	DiagrammeStatique * dg = fenetre->getNoyau()->getProjet()->getDiagrammeStatique();
	string s = pa->getNomComplet();

	fenetre->supprimerPaquetage( pa );
	cout << "SUPPRESSION (paquetage) " << s << endl;

	if ( noeud_pere == dg ) {
		if ( dg->supprimerObjetuml( (Objetuml *)pa ) == FALSE )
				QMessageBox::warning(0, "Erreur de suppression", "Impossible de supprimer ce paquetage dans le diagramme statique");
		else if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == pa) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
	} else {
		if ( ((Paquetage *)noeud_pere)->supprimerObjetuml( (Objetuml *)pa ) == FALSE )
				QMessageBox::warning(0, "Erreur de suppression", "Impossible de supprimer ce paquetage dans ce paquetage");
		else if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == pa) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
	}
}

void VisiteurSupprimer::traiterMethode(Methode *m)
{
	Noeud * noeud_pere = m->getPere();
	string s = m->grandeChaine();

	if ( ((Classe *)noeud_pere)->supprimerMethode(m) == FALSE )
		QMessageBox::warning(0,"Suppression d'une mthode",
			"Il est impossible de supprimer cette mthode");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == m) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (methode) " << s << endl;
	}
}

void VisiteurSupprimer::traiterAttribut(Attribut *attr)
{
	Noeud * noeud_pere = attr->getPere();
	string s = attr->grandeChaine();

	if ( ((Classe *)noeud_pere)->supprimerAttribut(attr) == FALSE )
		QMessageBox::warning(0,"Suppression d'un attribut",
			"Il est impossible de supprimer cet attribut");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == attr) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (attribut) " << s << endl;
	}
}

void VisiteurSupprimer::traiterHeritage(Heritage *h)
{
	Classe * classe = h->getClasseSource();
	string s = h->getNom();

	if ( classe->supprimerRelation(h) == FALSE )
		QMessageBox::warning(0,"Suppression d'une relation",
			"Il est impossible de supprimer cette relation");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == h) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (relation)" << s << endl;
	}
}

void VisiteurSupprimer::traiterUtilisation(Utilisation *u)
{
	Classe * classe = u->getClasseSource();
	string s = u->getNom();

	if ( classe->supprimerRelation(u) == FALSE )
		QMessageBox::warning(0,"Suppression d'une relation",
			"Il est impossible de supprimer cette relation");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == u) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (relation)" << s << endl;
	}
}

void VisiteurSupprimer::traiterAssociation(Association *assoc)
{
	Classe * classe = assoc->getClasseSource();
	string s = assoc->getNom();

	if ( classe->supprimerRelation(assoc) == FALSE )
		QMessageBox::warning(0,"Suppression d'une relation",
			"Il est impossible de supprimer cette relation");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == assoc) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (relation)" << s << endl;
	}
}

void VisiteurSupprimer::traiterAgregation(Agregation *ag)
{
	Classe * classe = ag->getClasseSource();
	string s = ag->getNom();

	if ( classe->supprimerRelation(ag) == FALSE )
		QMessageBox::warning(0,"Suppression d'une relation",
			"Il est impossible de supprimer cette relation");
	else {
		if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == ag) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
		cout << "SUPPRESSION (relation)" << s << endl;
	}
}

void VisiteurSupprimer::traiterRelationCommentaire(RelationCommentaire *rc)
{
	Commentaire * comm = rc->getCommentaire();
	string s = rc->getNom();

	comm->supprimerRelationCommentaire(rc);
	if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == rc) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
	
	cout << "SUPPRESSION (relation)" << s << endl;
}

void VisiteurSupprimer::traiterCommentaire(Commentaire *comm)
{
	std::vector<RelationCommentaire *> vct = comm->getvctRelCom();
	std::vector<RelationCommentaire *>::iterator itr;
	itr = vct.begin();
	while(itr != vct.end()){
		comm->supprimerRelationCommentaire((*itr));
		++itr;
	}
	Vue * v = (Vue*)comm->getPere();
	v->supprimerCommentaire(comm);
	if (fenetre->getBarreDeMenu()->getNoeudEnMemoire() == comm) fenetre->getBarreDeMenu()->setNoeudEnMemoire(NULL);
	
	cout << "SUPPRESSION (commentaire)" << comm << endl;

	delete(comm);
}

