#ifndef VISITEUR_VECTEUR
#define VISITEUR_VECTEUR

class FenetrePrincipale;

/** Cette classe reprsente le visiteur permettant de parcourir le diagramme statique afin
 * de mettre  jour les vecteurs de la fenetre principale. Elle sert notamment lors de l'import
 * de code source ou lors du chargement d'un fichier ".xml"
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class VisiteurVecteur : public VisiteurNoeud
{
	public:

		//! Constructeur
		VisiteurVecteur(FenetrePrincipale * fen);

		//! Destructeur
		virtual ~VisiteurVecteur() {}

		//! Cette mthode retourne le nombre d'objet qui ont t ajouts dans les vecteurs.
		int getNbAjout() { return nbAjout; }

		/** Cette mthode s'occupe de parcourir les "objetuml" 
		 * du diagramme statique afin de les traiter.
		 * \param DiagrammeStatique *dg : Reference sur le diagramme statique  traiter.
		 */
		void traiterDiagrammeStatique(DiagrammeStatique *dg);

		/** Cette mthode s'occupe de l'ajout d'un Paquetage dans son vecteur.
		 * Elle parcours aussi les objetuml de "pa" afin de les traiter.
		 * \param Paquetage *pa : Reference sur le paquetage  traiter.
		 */
		void traiterPaquetage(Paquetage *pa);

		/** Cette mthode s'occupe de l'ajout d'une classe dans son vecteur.
		 * Elle parcours aussi les relations de "cl" afin de les traiter.
		 * \param Classe *cl : Reference sur la classe  traiter.
		 */
		void traiterClasse(Classe *cl);

		/** Cette mthode s'occupe de l'ajout d'une Relation d'hritage dans 
		 * le vecteur de relations.
		 * \param Heritage * h : Reference sur la relation  traiter.
		 */
		void traiterHeritage(Heritage *h);

		/** Cette mthode s'occupe de l'ajout d'une Relation d'Utilisation dans 
		 * le vecteur de relations.
		 * \param Utilisation * u : Reference sur la relation  traiter.
		 */
		void traiterUtilisation(Utilisation *u);

		/** Cette mthode s'occupe de l'ajout d'une Relation d'Association dans 
		 * le vecteur de relations.
		 * \param Association * assoc : Reference sur la relation  traiter.
		 */
		void traiterAssociation(Association *assoc);

		/** Cette mthode s'occupe de l'ajout d'une Relation d'Agregation dans 
		 * le vecteur de relations.
		 * \param Agregation * ag : Reference sur la relation  traiter.
		 */
		void traiterAgregation(Agregation *ag);

	private:
		FenetrePrincipale * fenetre ;
		int nbAjout ;
};

#endif

