#ifndef WIDGETCENTRAL_H
#define WIDGETCENTRAL_H

#include <qsplitter.h>

#include <noynoyau.h>
#include <ihmarbre.h>
#include <ihmfenetrededessin.h>
#include <ihmmonscrollview.h>

/** Cette classe represente le widget central que l'on va inserer dans la fentre principale.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class WidgetCentral : public QSplitter
{
	Q_OBJECT

	public:

		/** Constructeur
		 * \param Noyau *n : Reference sur le noyau associ  l'application.
		 * \param Orientation o : Represente l'orientation du widget central (Horizontal ou Vertical).
		 * \param QWidget *parent : Represente le widget parent du widget central.
		 * \param const char *name : Represente le nom que l'on veut donner  notre widget central.
		 */
		WidgetCentral(Noyau * n, Orientation o,QWidget *parent=0, const char *name=0);

		//! Destructeur
		virtual ~WidgetCentral();


		/** Permet de recuperer une reference sur l'arbre de ce WidgetCentral.
		 * \return Une reference sur l'arbre de ce WidgetCentral.
		 */
		inline Arbre * getArbre() { return arbre ; }

		/** Permet de recuperer une reference sur la fentre de dessin de ce WidgetCentral.
		 * \return Une reference sur la fentre de dessin de ce WidgetCentral.
		 */		
		inline FenetreDeDessin * getFenetreDeDessin() { return fenetreDeDessin; }

		
	private:
		Arbre * arbre;
		FenetreDeDessin * fenetreDeDessin;
};

#endif

