#ifndef IHMZONEDESSIN_H
#define IHMZONEDESSIN_H

#include <noyvue.h>
#include <noyvisiteurnoeud.h>

#include <qpoint.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qwidget.h>

class QWidget;
class FenetrePrincipale;
class Vue;

/** Cette classe represente un editeur graphique capable de dessiner un graphe  partir des information du noyau.
 * @author F.Duchaussoy, N.Longuet, G.Prevot
 */

class ZoneDessin: public QWidget, public VisiteurNoeud
{
	Q_OBJECT

	public:

		/** Constructeur
		 * \param FenetrePrincipale *fen : rfrence sur la fenetre principale afin de
		 * communiquer avec le reste de l'ihm
		 * \param Vue *v : pointeur sur la vue  dessiner
		 * \param QWidget * : Represente le widget parent de la zone de dessin.
		 * \param const char *name : Represente le nom que l'on veut donner  notre zone de dessin.
		 */
		ZoneDessin(FenetrePrincipale * fen, Vue * v, QWidget *parent=0, const char *name=0);

		//! Destructeur
		virtual ~ZoneDessin();


		/** Accesseur  la variable prive "pinceau"
		 * \return retourne un pointeur sur le pinceau qui permet de dessiner la vue 
		 */
		QPainter * getPinceau(){ return pinceau; }

		//! mthode hrite de VisiteurNoeud permettant de traiter une vue
		void traiterVue( Vue *v );
		//! mthode hrite de VisiteurNoeud permettant de traiter un commentaire
		void traiterCommentaire(Commentaire * co);
		//! mthode hrite de VisiteurNoeud permettant de traiter un paquetage
		void traiterPaquetage(Paquetage *pa);
		//! mthode hrite de VisiteurNoeud permettant de traiter une classe
		void traiterClasse(Classe *cl);

		//! mthode hrite de VisiteurNoeud permettant de traiter une mthode
		void traiterMethode(Methode *m);
		//! mthode hrite de VisiteurNoeud permettant de traiter un attribut
		void traiterAttribut(Attribut *a);

		//! mthode hrite de VisiteurNoeud permettant de traiter un hritage
		void traiterHeritage(Heritage *h );
		//! mthode hrite de VisiteurNoeud permettant de traiter une utilisation
		void traiterUtilisation(Utilisation *u);
		//! mthode hrite de VisiteurNoeud permettant de traiter une association
		void traiterAssociation(Association *a);
		//! mthode hrite de VisiteurNoeud permettant de traiter une agrgation
		void traiterAgregation(Agregation *a);

		//! mthode surcharge permettant de redessiner la zone de dessin
		void paintEvent( QPaintEvent * e );
		//! mthode surcharge permettant de ragir  un bouton press
		void mousePressEvent ( QMouseEvent * e );
		//! mthode surcharge permettant de ragir  un bouton relach
		void mouseReleaseEvent ( QMouseEvent * e );
		//! mthode surcharge permettant de ragir  un double clic
		void mouseDoubleClickEvent ( QMouseEvent * e );

	private:
		bool positionnerRelation ( Relation * r );
		void intersections(float abs0, float ord0, float l0, float h0,
					float abs1, float ord1, float l1, float h1, QPoint buffer[2]);
		void dessinerCadre( ObjetAffichable * oa );
		bool getCoordonnees( Noeud * noeud , QPoint vecteur[2] );
		QSize getTaille();

		ObjetAffichable * objetSelectionne ;
		int typeSelection ;

		FenetrePrincipale * fenetre ;
		QPainter *pinceau ;
		Vue * vue ;

		std::vector<ObjetAffichable*> vct;
		std::vector<ObjetAffichable*>::iterator it;
		int x0,x1,y0,y1,xTemp,yTemp;
	

	public slots:
		//! Ce slot est appel pour enregistrer une vue en Pixmap dans un fichier.
		void enregistrerVue();
};

#endif

