/************************************************/
/*      N.Douay - C.Lucas                       */
/************************************************/
/*      N.Douay - C.Lucas                       */
/************************************************/
/*     mer fv 13 13:36:35 CET 2002             */
/************************************************/
/*     mer fv 13 13:36:35 CET 2002             */
/************************************************/
/*      Numero de version et commentaire        */
/*      0.1     Cration                        */
/************************************************/
/*      noydiagrammestatique.h                            */
/************************************************/

#ifndef NOYDIAGRAMMESTATIQUE_H
#define NOYDIAGRAMMESTATIQUE_H
#include <iostream>
#include <vector.h>

#include <noynoeud.h>
#include <noyvue.h>
#include <noypaquetage.h>
#include <noyobjetuml.h> 

/** Cette classe dfinit un diagramme statique au sens UML mais aussi au sens graphiue du terme
* @author clucas-ndouay
*/

class DiagrammeStatique:public Noeud{

private:
	//! vecteur contenant la liste des vues
	std::vector<Vue *> vctVue;
	//! vecteur contenant la liste des objets UML
	std::vector<Objetuml*> vctObjetuml;

public:
	//! Constructeur utilis pour crer un noeud diagramme statique avec comme nom "Diagramme statique"
	DiagrammeStatique():Noeud(0,"Diagramme statique"){}
	//! Constructeur par dfaut	
	DiagrammeStatique(Noeud* pere,const string &nom);
	//! Destructeur
	virtual ~DiagrammeStatique();
	
	/** Cette mthode permet d'accepter un visiteur
	* \param VisiteurNoeud *v : visiteur  accepter
	*/
	virtual void accepter(VisiteurNoeud *v);

	/**Cette fonction permet d'ajouter un objet uml au diagramme
	* \param Objetuml *o : pointeur sur un objet uml (ex: paquetage)
	* \return La fonction retourne "true" si l'objet est bien ajout et "false" sinon
	*/
	bool 		ajouterObjetuml(Objetuml *o);
	
	/**Cette mthode fait le contraire de la prcdente
	* \param Objetuml *o : pointeur sur l'objet uml 
	* \return La fonction retourne "true" si l'objet est bien supprim et "false" sinon
	*/
	bool 		supprimerObjetuml(Objetuml *o);
	
	/**
	 * Cette mthode permet d'enlever un objetuml du vecteur d'objet
	 * \param l'objet que l'on souhaite supprimer.
	 */
	bool 	enleverObjetuml(Objetuml* ob);

	/**Cette mthode permet d'ajouter une vue au diagramme
	* \param Vue *v : pointeur sur une vue
	* \return le retour est un boolen qui vaut "true" si l'ajout c'est bien pass
	* et "false" sinon
	*/
	bool 		ajouterVue(Vue *v);
	
	/**Cette mthode est l'oppos de la prcdente
	* \param Vue *v : pointeur sur la vue  supprimer
	* \return le retour es un boolen qui vaut "true" si la suppression  c'est effectue
	* et "false" sinon
	*/
	bool 		supprimerVue(Vue *v);

	/** Cette mthode permet de visiter une vue c'est  dire de la parcourir
	* \param VisiteurNoeud *v : visiteur permettant de parcourir la vue
	*/
	void 		visiterVue(VisiteurNoeud *v);

/*	Objetuml* 	possedeObjetuml( const string & v );
	Objetuml* 	possedeObjetuml( Objetuml *o );*/

	/** Cette mthode permet de savoir si une vue existe
	* \param const string & v : chane de caractres contenant le nom de la vue recherche
	* \return elle retourne un pointeur sur une vue
	*/
	Vue*	 	possedeVue( const string & v );

	/** Cette mthode fait comme la prcdente
	* \param Vue *v : pointeur sur la vue recherche
	* \return elle retourne un pointeur sur la vue recherche
	*/
	Vue*	 	possedeVue( Vue *v );

	/**Cette fonction permet d'ajouter un paquetage au diagramme
	* \param constr string & nom : chane de caractres reprsentant le nom du paquetage  ajouter
	* \return La fonction retourne "true" si l'objet est bien ajout et "false" sinon
	*/
	bool		ajouterPaquetage( const string & nom );
	
	/**Cette fonction permet de supprimer un paquetage du diagramme
	* \param Paquetage *p : pointeur sur le paquetage  supprimer
	* \return La fonction retourne "true" si l'objet est bien supprimer et "false" sinon
	*/
	bool		supprimerPaquetage( Paquetage *p );
	
	/**Cette fonction permet d'ajouter une classe au diagramme
	* \param constr string & nom : chane de caractres reprsentant le nom de la classe  ajouter
	* \return La fonction retourne "true" si l'objet est bien ajout et "false" sinon
	*/
	bool		ajouterClasse( const string & nom, const string &visibilite);
	
	/**Cette fonction permet de supprimer une classe du diagramme
	* \param Paquetage *p : pointeur sur la classe  supprimer
	* \return La fonction retourne "true" si l'objet est bien supprimer et "false" sinon
	*/
	bool		supprimerClasse( Classe *c );	


	/**Cette fonction permet d'ajouter une classe au diagramme
	* \param constr string & nom : chane de caractres reprsentant le nom de la classe  ajouter
	* \return La fonction retourne "true" si l'objet est bien ajout et "false" sinon
	*/
	bool		ajouterInterface( const string & nom, const string &visibilite);
	
	/**Cette fonction permet de supprimer une classe du diagramme
	* \param Paquetage *p : pointeur sur la classe  supprimer
	* \return La fonction retourne "true" si l'objet est bien supprimer et "false" sinon
	*/
	bool		supprimerInterface( Classe *c );	


	/**Cette fonction permet d'ajouter une vue au diagramme
	* \param constr string & nom : chane de caractres reprsentant le nom de la vue  ajouter
	* \return La fonction retourne "true" si l'objet est bien ajout et "false" sinon
	*/
	bool		ajouterVue( const string & nom );

	/**Cette fonction permet de supprimer une vue du diagramme
	* \param Paquetage *p : pointeur sur la vue  supprimer
	* \return La fonction retourne "true" si l'objet est bien supprimer et "false" sinon
	*/
	bool		supprimerVue( const string & nom );

	// Primitives noyau
	//void		visiterPaquetage(VisiteurNoeud *v);
	//void		visiterClasse(VisiteurNoeud *v);
	
	/** Cette fonction permet de parcourir un objet UML
	* \param VisiteurNoeud *v : visiteur permettant de parcourir l'objet UML
	*/
	void		visiterObjetuml(VisiteurNoeud *v);
	
	/** Cette fonction permet de parcourir les fils du diagramme
	* \param VisiteurNoeud *v : visiteur permettant de parcourir les fils du diagramme
	*/
	void		visiterFils(VisiteurNoeud *v);

	/** Cette mthode retourne le nombre d'objet UML prsent dans le vecteur
	* \return elle retourne la taille du vecteur d'objets UML
	*/
	int 		NbObjetuml();
	
	/** Cette mthode retourne le nombre de vue prsent dans le vecteur
	* \return elle retourne la taille du vecteur de vue
	*/
	int 		NbVue();

	/** Cette mthode permet de rcuprer le vecteur de vues
	* \return elle retourne le vecteur de vue du diagramme statique
	*/
	std::vector<Vue *> getVctVue();
	
	/** Cette mthode permet de rcuprer le vecteur d' objets UML
	* \return elle retourne le vecteur d' objets UML du diagramme statique
	*/
	std::vector<Objetuml *> getVctObjetuml();

//	string getNomComplet() ;

};
#endif
