/************************************************/
/*      N.Douay - C.Lucas                       */
/************************************************/
/*      N.Douay - C.Lucas                       */
/************************************************/
/*     mer fv 13 13:36:40 CET 2002             */
/************************************************/
/*     mer fv 13 13:36:40 CET 2002             */
/************************************************/
/*      Numero de version et commentaire        */
/*      0.1     Cration                        */
/************************************************/
/*      noyprojet.h                             */
/************************************************/

#ifndef NOYPROJET_H
#define NOYPROJET_H
#include <iostream>

#include <noynoeud.h>
#include <noyusecase.h>
#include <noycollaboration.h>
#include <noydiagrammestatique.h>
#include <noynoyau.h>

/** Cette classe dcrit un projet au sens large du terme
* Le Projet est le premier noeud de l'arbre
* @author clucas-ndouay
*/

class Projet:public Noeud{

private:
	//! reprsente le noeud des cas d'utilisation dans l'arbre
	UseCase 		u;
	//! reprsente le noeud des diagrammes de collaboration dans l'arbre
	Collaboration 		c;
	//! reprsente le noeud des diagrammes statiques de classe
	DiagrammeStatique 	d;
	//! nom du projet
	string _nom; /* demander  syj pourquoi on avait enlever le nom ?!! */
	//! version du projet
	string _version;
	//! commentaires lis au projet
	string _commentaire;
	//! auteur du projet
	string _auteur;
	//! pointeur sur le noyau du projet en cours
	Noyau	*_n ;
	//! booleen precisant le langage
	bool _langage;
	//! boolen prcisant si la barre ajouter est affiche
	bool _boolaj;
	//! boolen prcisant si la barre dition est affiche
	bool _booled;
	//! boolen prcisant si la barre projet est affiche
	bool _boolpro;
	//! boolen pour les noms longs
	bool _boolnom;
	//! boolen pour les accesseurs
	bool _boolacc;

public :
	/** Constructeur
	* \param string nom : nom donn par l'utilisateur au projet
	* \param Noyau *n : pointeur sur le noyau du projet cr
	*/
	Projet(const string &nom , Noyau *n ,bool b1=true, bool b2=false, bool b3=false, bool b4=true, bool b5=true, bool b6=false);
// A GARDER :	Projet( ):Noeud(0,"Projet"){}
//	Projet(Noeud* pere, const string &nom, Noyau *n);
	//! Destructeur
	virtual ~Projet();
	/** mthode permettant d'accepter le visiteur
	* \param VisiteurNoeud* v : pointeur sur le visiteur
	*/
	virtual void accepter(VisiteurNoeud* v);
	
	/** Cette mthode permet de rcuprer un pointeur sur le diagramme statique
	* \return elle retourne un pointeur sur un objet de type DiagrammeStatique
	*/
	DiagrammeStatique* getDiagrammeStatique();
	
	/** Cette mthode permet de rcuprer un pointeur sur les cas d'utilisations
	* \return elle retourne un pointeur sur un objet de type UseCase
	*/
	UseCase* getUseCase();
	
	/** Cette mthode permet de rcuprer un pointeur sur les diagrammes de collaboration
	* \return elle retourne un pointeur sur un objet de type Collaboration
	*/
	Collaboration* getCollaboration();
	
	/** Cette mthode permet de savoir quelle est la version du projet en cours
	* \return elle retourne une chane de caractres contenant la version
	*/
	string getVersion();
	
	/** Cette mthode permet de fixer la version du projet
	* \param string version : chane de caractres contenant la version
	*/
	void setVersion(const string &version);
	
	/** Cette mthode permet d'obtenir le nom de l'auteur du projet
	* \return elle retourne une chane de caractres contenant le (ou les) nom(s) de l' (ou des) auteur(s)
	*/
	string getAuteur();
	
	/** Cette mthode permet de fixer le (ou les) nom(s) de l' (ou des) auteur(s) du projet
	* \param string version : chane de caractres contenant le (ou les) nom(s) de l' (ou des) auteur(s)
	*/
	void setAuteur(const string &auteur);
	
	/** Cette mthode permet d'obtenir les divers commentaires sur le projet
	* \return elle retourne une chane de caractres contenant le (ou les) commentaire(s)
	*/
	string getCommentaire();
	
	/** Cette mthode permet de fixer le (ou les) commentaire(s) sur le projet
	* \param string version : chane de caractres contenant le (ou les) commentaire(s)
	*/
	void setCommentaire(const string &commentaire);

	/** Cette mthode est un accesseur au boolen "langage"
	 * \return le boolen voulu
	*/ 
	bool getLangage();
	
	/** Cette mthode est un accesseur au boolen "boolaj"
	 * \return le boolen voulu
	*/
	bool getAjouter();

	/** Cette mthode est un accesseur au boolen "booled"
	 * \return le boolen voulu
	*/
	bool getEdition();
	
	/** Cette mthode est un accesseur au boolen "boolpro"
	 * \return le boolen voulu
	*/
	bool getPro();

	/** Cette mthode est un accesseur au boolen "boolnom"
	 * \return le boolen voulu
	*/
	bool getNomLong();

	/** Cette mthode est un accesseur au boolen "boolacc"
	 * \return le boolen voulu
	*/
	bool getAccesseur();

	/** Cette mthode fixe un paramtre prfrence
	 * \param un boolen correspondant au boolen  changer
	*/ 
	void setLangage(bool b);

	/** Cette mthode fixe un paramtre prfrence
	 * \param un boolen correspondant au boolen  changer
	*/ 
	void setAjouter(bool b);

	/** Cette mthode fixe un paramtre prfrence
	 * \param un boolen correspondant au boolen  changer
	*/ 
	void setEdition(bool b);

	/** Cette mthode fixe un paramtre prfrence
	 * \param un boolen correspondant au boolen  changer
	*/ 
	void setPro(bool b);
	
	/** Cette mthode fixe un paramtre prfrence
	 * \param un boolen correspondant au boolen  changer
	*/ 
	void setNomLong(bool b);
	
	/** Cette mthode fixe un paramtre prfrence
	 * \param un boolen correspondant au boolen  changer
	*/ 
	void setAccesseur(bool b);
};
#endif
