/************************************************/
/*      N.Douay - C.Lucas                       */
/************************************************/
/*      N.Douay - C.Lucas                       */
/************************************************/
/*     mer fv 13 13:36:44 CET 2002             */
/************************************************/
/*     mer fv 13 13:36:44 CET 2002             */
/************************************************/
/*      Numero de version et commentaire        */
/*      0.1     Cration                        */
/************************************************/
/*      noyvue.h                            */
/************************************************/

#ifndef NOYVUE_H
#define NOYVUE_H
#include <iostream>
#include <vector>

#include <noynoeud.h>
#include <noyobjetaffichable.h>
#include <noyclasse.h>
#include <noypaquetage.h>
#include <noycommentaire.h>

/**Cette classe est l'implmentation d'une vue au sens IHM.
* En effet elle n'a aucun lien direct avec la reprsentation UML
* si ce n'est qu'elle permet de visualiser les diffrents diagrammes
* raliss par l'utilisateur.
* @author clucas-ndouay
*/
class Vue:public Noeud{

private:
	std::vector<ObjetAffichable*> vctObjetAffichable ;
public:

	/**Constructeur d'une vue
	* \param Noeud *pere : pointeur sur le noeud de l'arborescence qui est le pere direct de la vue
	* \param const string &nom: nom donn par l'utilisateur  la nouvelle vue
	*/
	Vue(Noeud* pere,const string &nom);

	/**Destructeur d'une vue
	*/
	virtual ~Vue();
	
	/**Cette mthode permet d'ajouter un nouvel objet affichable  la vue
	* \param ObjetAffichable *o : pointeur sur l'objet affichable  ajouter
	* \return le retour est "true"  si l'objet a bien t ajout et "false" sinon
	*/
	bool	ajouterObjetAffichable( ObjetAffichable *o );

	
	/**
	 * Cette mthode permet d'ajouter un lment de modlisation dans une vue.
	 * \param un lment de modlisation
	 * \param coordonnes (x,y)
	 * \param largeur et hauteur
	 * \param visibilit de l'objet dans l'aire de dessin
	 */
	bool 	ajouterElementDeModelisation( ElementDeModelisation *e , int x , int y, int w, int h , bool visible );

	/**Cette mthode permet d'ajouter une classe  la vue
	* \param Classe *c : pointeur sur la classe  ajouter
	* \return le retour est "true"  si la classe a bien t ajout et "false" sinon
	*/
	bool	ajouterClasse( Classe *c );

	/**Cette mthode permet d'ajouter un nouveau paquetage  la vue
	* \param Paquetage *p : pointeur sur le paquetage  ajouter
	* \return le retour est "true"  si le paquetage a bien t ajout et "false" sinon
	*/
	bool	ajouterPaquetage( Paquetage *p );
	
	/**Cette mthode permet d'ajouter une nouvelle relation  la vue
	* \param ObjetAffichable *o : pointeur sur la relation  ajouter
	* \return le retour est "true"  si la relation a bien t ajout et "false" sinon
	*/
	bool	ajouterRelation( Relation *r );


	/**Fonction permettant d'enlever un objet d'une vue
	* \param ObjetAffichable *o : pointeur sur l'objet  supprimer de la vue
	* \return le retour est "true" si l'objet a t supprimer et "false" sinon
	*/
	bool	supprimerObjetAffichable( ObjetAffichable *o );

	/**Cette mthode permet de supprimer une classe de la vue
	* \param Classe *c : pointeur sur la classe  supprimer
	* \return le retour est "true"  si l'objet a bien t supprim et "false" sinon
	*/
	bool	supprimerClasse( Classe *c );

	/**Cette mthode permet de supprimer un paquetage de la vue
	* \param Paquetage *p : pointeur sur le paquetage  supprimer
	* \return le retour est "true"  si l'objet a bien t supprim et "false" sinon
	*/
	bool	supprimerPaquetage( Paquetage *p );

	/**Cette mthode permet de supprimer une relation de la vue
	* \param Relation *r : pointeur sur la relation  supprimer
	* \return le retour est "true"  si l'objet a bien t supprim et "false" sinon
	*/
	bool	supprimerRelation( Relation *r );

	/**Cette mthode permet d'accepter un visiteur afin de parcourir la vue
	* \param VisiteurNoeud* v : pointeur sur le visiteur
	*/
	virtual void accepter(VisiteurNoeud* v);
	/**cette fonction permet de savoir si un objet appartient  la vue
	* \param void *o : un objet quelconque
	* \return le retour est "true" si l'objet est dans la vue et "false" sinon
	*/
	bool possedeObjet( void *o );

	/**Cette mthode permet de rcuprer un vecteur contenant la liste des objets affichables dans la vue
	 \return le retour est un vecteur d'objet affichable
	*/
	std::vector<ObjetAffichable*> getvctObjetAffichable();

	/**
	 * Cette mthode permet d'ajouter un commentaire dans une vue
	 * \param le texte qui se trouve dans le commentaire UML.
	 */
	bool ajouterCommentaire( const string & c );

	/**
	 * Cette mthode permet de supprimer un commentaire de la vue.
	 * \param Commentaire *c : commentaire  supprimer
	 * \return elle retourne "true" si la suppression  lieu et "false" sinon
	 */
	bool supprimerCommentaire( Commentaire *c ) ;

	/**Cette mthode permet d'ajouter une nouvelle relation commentaire  la vue
	* \param Relation *r : pointeur sur la relation  ajouter
	* \return le retour est "true"  si la relation a bien t ajout et "false" sinon
	*/
	bool	ajouterRelationCommentaire( Relation *r );

	/**
	 * Cette mthode permet de supprimer une relation commentaire de la vue.
	 * \param Relation *r : relation commentaire  supprimer
	 * \return elle retourne "true" si la suppression  lieu et "false" sinon
	 */
	bool supprimerRelationCommentaire(Relation *r);
};
#endif
