#ifndef NOEUDSYNTAXIQUE_H
#define NOEUDSYNTAXIQUE_H

#include <iostream>
#include <string>

class VisiteurNoeudSyntaxique;
class ContexteSemantique;

/**
 *Un noeud syntaxique correspond  un element de la pile des symboles.<br>
 *@author syj.
 */

class NoeudSyntaxique {
		int _type;
		ContexteSemantique *_csPere;
		string _nom;
		std::vector<string> vctAlias;
	public:
		NoeudSyntaxique(const NoeudSyntaxique &ns);
		NoeudSyntaxique(ContexteSemantique *csPere,const string &nom,int type);
		virtual ~NoeudSyntaxique();
		
		
		/**
		 *Recherche d'un NoeudSyntaxique 
		 *@param s le nom du NoeudSyntaxique.
		 *@return Le NoeudSyntaxique recherch ou NULL.
		 */
		virtual NoeudSyntaxique *possedeNS(const string &s);
		
		/**
		 *Ajouter un alias.
		 */
		void ajouterAlias(const string &s);
		 
		//! Obtenir le type du lexeme.
		int getType();		
		
		//! Definir le type du lexeme.
		void setType(int type);
		
		/**
	 	*Visiter le noeud syntaxique.
		 *@param v le visiteur
		 */
		virtual void accepterVNS(VisiteurNoeudSyntaxique *vns) {};	
		
		//! Obtenir le nom principal du noeud syntaxique.
		string getNom();
		
		//! Definir le nom principal du noeud syntaxique.
		void setNom(string &nom);
		
		//! Obtenir le contexte semantique pere.
		ContexteSemantique *getCsPere();
		
		//! Definir le contexte semantique pere.
		void setCsPere(ContexteSemantique *csPere);
		
		//! Renvoie le nom complet d'un noeud syntaxique.
		virtual string getNomComplet();	

		/**
		 * Remplace un noeud syntaxique par un autre.
		 * @param ns l'ancien noeud syntaxique
		 * @param ns2 le remplaant.
		 */
		virtual void remplacerNS(NoeudSyntaxique *ns,NoeudSyntaxique *ns2) {}

		
		/**
		 *Teste l'appartenance d'un NoeudSyntaxique dans un vector.
		 *@param vct le vector qui est test.
		 *@param ns le NoeudSyntaxique recherch.
		 *@return le NoeudSyntaxique ns ou NULL s'il n'a pas t trouv.
		 */
		static NoeudSyntaxique *nsAppartientAuVector(std::vector<NoeudSyntaxique *> &vct,NoeudSyntaxique *ns);
		
		/**
		 *Teste l'existence d'un NoeudSyntaxique de mme dans un vector.
		 *@param vct le vector qui est test.
		 *@param s le nom recherch.
		 *@return le NoeudSyntaxique si il est trouv sinon elle renvoie NULL.
		 */
		static NoeudSyntaxique *nomAppartientAuVector(std::vector<NoeudSyntaxique *> &vct,const string &s);
		
		/**
		 *Visite le vector de NoeudSyntaxique.
		 *@param vct le vector qui est visit.
		 *@param vns l'instance du visiteur.
		 */
		static void visiterVectorNoeudSyntaxique(std::vector<NoeudSyntaxique *> &vct,VisiteurNoeudSyntaxique *vns);		
		
		/**
		 * Remplacer un noeud syntaxique dans un vector de NoeudSyntaxique*
		 */
		static void remplacerDansLeVector(std::vector<NoeudSyntaxique *> &vct,NoeudSyntaxique *ns,NoeudSyntaxique *ns2);

		/**
		 * Supprimer les references  un noeud syntaxique dans un vector de NoeudSyntaxique*
		 * @param ns le noeud  supprimer
		 */
		static void NoeudSyntaxique::supprimerDansLeVector(std::vector<NoeudSyntaxique *> &vct,NoeudSyntaxique *ns); 
	
	};
#endif
 
