#include<parparser.h>

NsClasse::NsClasse(ContexteSemantique *csPere ,const string &nom,int type) 
	: ContexteSemantique(csPere,nom,type)
	{
	_cls = NULL;
	_visibilite = DF_PRIVATE;
	declare = false;
	nomCode = nom;
	}
	
NsClasse::~NsClasse() {
	cout << "Appelle du destructeur NsClasse" << endl;
	// RAPPEL: On libere la memoire des noeuds syntaxique que si on est leur pere !	
	}

NoeudSyntaxique *NsClasse::estPrivate(NoeudSyntaxique *ns) {
	return nsAppartientAuVector(vctPrivateNs,ns);
	}
		
NoeudSyntaxique *NsClasse::estPrivate(const string &s) {
	return nomAppartientAuVector(vctPrivateNs,s);
	}
		
NoeudSyntaxique *NsClasse::estProtected(NoeudSyntaxique *ns) {
	return nsAppartientAuVector(vctProtectedNs,ns);	
	}
		
NoeudSyntaxique *NsClasse::estProtected(const string &s) {
	return nomAppartientAuVector(vctProtectedNs,s);
	}
		
NoeudSyntaxique *NsClasse::estPublic(NoeudSyntaxique *ns) {
	return possedeVctNS(ns);
	}
		
NoeudSyntaxique *NsClasse::estPublic(const string &s) {
	return possedeVctNS(s);
	}
	

int NsClasse::getVisibilite() {
	return _visibilite;
	}


int NsClasse::getVisibilite(NoeudSyntaxique *ns) {
	if(estPublic(ns) == ns) return DF_PUBLIC;
	if(estProtected(ns) == ns) return DF_PROTECTED;
	return DF_PRIVATE;
	}
		
void NsClasse::setVisibilite(int visibilite) {
	_visibilite = visibilite;
	}
	
	
NoeudSyntaxique *NsClasse::possedeIntimeNS(const string &s) {
	NoeudSyntaxique *ns;
	
	// Verifie les variables apparaissant globalement.
	ns = possedeNS(s);
	if(ns != NULL) return ns;

	// Verifie les variables apparaissant protected puis private.
	ns = nomAppartientAuVector(vctProtectedNs,s);
	if(ns != NULL)return ns;
	ns = nomAppartientAuVector(vctPrivateNs,s);	
	if(ns != NULL)return ns;
		
	std::vector<NsClasse *>::iterator it = vctHeriteDe.begin();
	while(it != vctHeriteDe.end()) {
		ns = ((NsClasse *)*it)->possedeIntimeNS(s);
		if(ns != NULL) return ns;
		++it;
		}

	return NULL;
	}


NoeudSyntaxique *NsClasse::possedeNS(const string &s) {
	if(getNom() == s) return this;
	
	NoeudSyntaxique *ns;
	if((ns = possedeVctNS(s)) != NULL) return ns;
	
	return NULL;
	}
	
void NsClasse::accepterVNS(VisiteurNoeudSyntaxique *v) {
	v->traiterNsClasse(this);
	}

void NsClasse::ajouterNS(NoeudSyntaxique *ns) {
	switch(_visibilite) {
		case DF_PUBLIC:
			ajouterVctNS(ns);
			break;
			
		case DF_PROTECTED:
			vctProtectedNs.push_back(ns);
			break;
			
		case DF_PRIVATE:
			vctPrivateNs.push_back(ns);
			break;
		}
	}
	
void NsClasse::ajouterClasseMere(NsClasse *nsc,int visibilite) {
	vctHeriteDe.push_back(nsc);
	/*
	switch(visibilite) {
		case DF_PUBLIC:
			vctNs.insert(vctNs.end(),nsc->vctNs.begin(),nsc->vctNs.end());
			vctProtectedNs.insert(vctProtectedNs.end(),nsc->vctProtectedNs.begin(),nsc->vctProtectedNs.end());
			break;
		case DF_PROTECTED:
			vctProtectedNs.insert(vctNs.end(),nsc->vctNs.begin(),nsc->vctNs.end());
			vctPrivateNs.insert(vctProtectedNs.end(),nsc->vctProtectedNs.begin(),nsc->vctProtectedNs.end());
			break;
		case DF_PRIVATE:
			vctPrivateNs.insert(vctPrivateNs.end(),nsc->vctNs.begin(),nsc->vctNs.end());			
			vctPrivateNs.insert(vctPrivateNs.end(),nsc->vctProtectedNs.begin(),nsc->vctProtectedNs.end());
		}
	*/
	}

void NsClasse::visiterClasseMere(VisiteurNoeudSyntaxique *vns) {
	std::vector<NsClasse *>::iterator it = vctHeriteDe.begin();
	while(it != vctHeriteDe.end()) {
		((NsClasse *)*it)->accepterVNS(vns);
		++it;
		}
	}	


void NsClasse::visiterNS(VisiteurNoeudSyntaxique *vns) {
	visiterNS(vns,DF_PUBLIC);
	visiterNS(vns,DF_PROTECTED);
	visiterNS(vns,DF_PRIVATE);
	}
		
void NsClasse::visiterNS(VisiteurNoeudSyntaxique *vns,int visibilite) {
	switch(visibilite) {
		case DF_PUBLIC:
			visiterVctNS(vns);
			break;
		case DF_PROTECTED:
			visiterVectorNoeudSyntaxique(vctPrivateNs,vns);
			break;
		case DF_PRIVATE:
			visiterVectorNoeudSyntaxique(vctProtectedNs,vns);
			break;
		}
	}
	
void NsClasse::setClasse(Classe *cls) {
	_cls = cls;
	}
		
Classe *NsClasse::getClasse() {
	return _cls;
	}



class VnsIsInterface : public VisiteurNoeudSyntaxique {
	public:
		bool isInterface;
		VnsIsInterface() {isInterface = true;}
		
		void traiterNsAttribut(NsAttribut *nsa) {
			isInterface = false;
			}
		
		void traiterNsMethode(NsMethode *nsm) {
			if(nsm->getAbstract() == false) {
				isInterface = false; 
				}
			}
	};

bool NsClasse::isInterface() {
	VnsIsInterface vns;
	visiterNS(&vns);	
	return vns.isInterface;
	}

void NsClasse::remplacerNS(NoeudSyntaxique *ns,NoeudSyntaxique *ns2) {
	remplacerDansLeVector(vctProtectedNs,ns,ns2);
	remplacerDansLeVector(vctPrivateNs,ns,ns2);

	std::vector<NsClasse *>::iterator it = vctHeriteDe.begin();
	while(it != vctHeriteDe.end()) {
		if(*it == ns) {
			vctHeriteDe.erase(it);
			vctHeriteDe.push_back((NsClasse*)ns2);
			it = vctHeriteDe.begin();
			}
		++it;
		}

	ContexteSemantique::remplacerNS(ns,ns2);
	}

void NsClasse::supprimerNS(NoeudSyntaxique *ns) {
	supprimerDansLeVector(vctProtectedNs,ns);
	supprimerDansLeVector(vctPrivateNs,ns);
	supprimerVctNS(ns);
	}

