#ifndef NSCLASSE_H
#define NSCLASSE_H

#define DF_PUBLIC 0
#define DF_PROTECTED 1
#define DF_PRIVATE 2

class Classe;

/**
 * NoeudSyntaxique representant une classe.
 * @author syj
 */
class NsClasse : public ContexteSemantique {	
	private:
		std::vector<NsClasse *> vctHeriteDe;
		std::vector<NoeudSyntaxique *>vctFriend;
		std::vector<NoeudSyntaxique *>vctProtectedNs;
		std::vector<NoeudSyntaxique *>vctPrivateNs;
		int _visibilite;
		
		Classe *_cls;
		
	protected:
		//! Test ns est un noeud private.
		NoeudSyntaxique *estPrivate(NoeudSyntaxique *ns);
		
		//! Test si s est un noeud private.
		NoeudSyntaxique *estPrivate(const string &s);			
		
		//! Test si ns est un noeud protected.
		NoeudSyntaxique *estProtected(NoeudSyntaxique *ns);
		
		//! Test si s est un noeud protected.
		NoeudSyntaxique *estProtected(const string &s);
		
		//! Test ns est un noeud public.
		NoeudSyntaxique *estPublic(NoeudSyntaxique *ns);
		
		//! Test s est un noeud public.
		NoeudSyntaxique *estPublic(const string &s);
		
		
	public:
		bool declare;
		string nomCode;
		std::vector<string> currentImport;
		
		NsClasse(ContexteSemantique *csPere, const string &nom,int type);
		virtual ~NsClasse();
		
		
		NoeudSyntaxique *possedeProtectedNs(NoeudSyntaxique *ns);
		 
		/**
		 *Obtenir la visibilit des noeuds syntaxique qui vont tre ajout.
		 *@return un nombre compris entre 0 et 2 (0 public, 1 protected, 2 private).
		 */
		int getVisibilite();
		
		//! Renvoie la visibilit d'un NoeudSyntaxique.
		int getVisibilite(NoeudSyntaxique *ns);

		
		/**
		 *Definir la visibilit des noeuds syntaxique qui vont tre ajout.
		 *@param la valeur en parametre doit tre soit DF_PUBLIC, DF_PROTECTED, ou DF_PRIVATE.
		 */
		void setVisibilite(int visibilite);
		
		
		/**
		 *Ajoute un noeud syntaxique celon la visibilite actuel.
	 	*@param ns le noeud a ajouter.
		 */
		virtual void ajouterNS(NoeudSyntaxique *ns);
			
		/**
		 *Visiter le noeud syntaxique.
		 *@param v le visiteur
		 */
		virtual void accepterVNS(VisiteurNoeudSyntaxique *vns);

		/**
		 *Ajouter une classe parente.
		 *@param nsc la classe mere  ajouter.
		 */			
		void ajouterClasseMere(NsClasse *nsc,int visibilite);
		
		/**
		 *Cette methode apppelle le VisiteurNoeudSyntaxique.
		 *@param vns le visiteur utilis.
		 */
		void visiterClasseMere(VisiteurNoeudSyntaxique *vns);
		
		/**
		 *Effectuer accepterVNS(vns) pour chacun de ses fils.
		 *Attention: les classes parentes ne sont pas considr comme des fils !
		 *@param vns le visiteur utilis.
	 	*/
		virtual void visiterNS(VisiteurNoeudSyntaxique *vns);
		
		/**
		 *Permet de visiter que les elements d'une visibilit donn.
		 *@param vns le visiteur utilis.
	 	*/
		virtual void visiterNS(VisiteurNoeudSyntaxique *vns,int visibilite);

		
		/**
		 *Recherche d'un NoeudSyntaxique dans son contexte.
		 *@param s le nom du NoeudSyntaxique.
		 *@return Le NoeudSyntaxique recherch ou NULL.
		 */
		virtual NoeudSyntaxique *possedeNS(const string &s);
	
		/**
		 *Recherche dans les proprits des classes mre et dans les champs protected et private en + de ceux parcourues
		 *par possedeNS
		 *@param s le nom du NoeudSyntaxique.
		 *@return Le NoeudSyntaxique recherch ou NULL.
		 */
		NoeudSyntaxique *possedeIntimeNS(const string &s); 
		
		/**
		 *Definir la classe associ.
		 *@param cls un pointeur sur classe.
		 */
		void setClasse(Classe *cls);
		
		/**
		 *Obtenir la classe associ.
		 */
		Classe *getClasse();

		/**
		 *Obtenir si la classe est une interface.<br>
		 *Note: Une interface est une classe sans attribut et ne possedant que des methodes virtual.
		 */
		bool isInterface();

		/**
		 * Remplace un noeud syntaxique par un autre.
		 * @param ns l'ancien noeud syntaxique
		 * @param ns2 le remplaant.
		 */
		virtual void remplacerNS(NoeudSyntaxique *ns,NoeudSyntaxique *ns2);

		/**
		 * Supprimer un noeud syntaxique
		 * @param ns le noeud  supprimer
		 */
		virtual void supprimerNS(NoeudSyntaxique *ns);

};

#endif
