#ifndef NSMETHODE_H
#define NSMETHODE_H

class Methode;

/**
 * NsMethode decrit un methode au sens UML. Cette classe fortement <br>
 * li  la classe NOYAU::Methode car NsMethode correspond  un adapteur <br>
 * afin que la classe Methode puisse repondre au exigence de fonctionnement du parser.<br>
 * tandis qu'une classe sera ajout  la fois au paquetage du noyau, et au contexte semantique.<br>
 *@author syj
 */
class NsMethode : public ContexteSemantique {
	//! La liste des parametres.
	std::vector<NsAttribut *> vctParametre;
	
	//! Le type de retour de la methode.
	Type _type;
	
	Methode *_methode;

	/**
	 * Valeur indiquant si la methode est abstraite.<br>
	 * Par exemple:<br>
	 * En java: <br>
	 * La methode se trouve tre dans une interface ou tre prced du mot abstract. <br>
	 * En C++:<br>
	 * La methode est precede d'un =0 indiquant cette methode doit tre redefinit<br>
	 * dans les classes filles.
	 */
	bool _Abstract;
	
public:
	NsMethode(ContexteSemantique *csPere,const string &nom,int type);
	
	virtual ~NsMethode();
	
	/**
	 *Visiter le noeud syntaxique.
	 *@param v le visiteur
	 */
	virtual void accepterVNS(VisiteurNoeudSyntaxique *v);
	
	/**
	 *Cette methode visiterNS sur le ContexteSemantique pass en parametre
	 *pour rcuperer que les NsAttributs qui deviendront ses parametres.
	 *@param cs le contexte semantique contenant des NsAttribut
	 */
	void ajouterParametre(ContexteSemantique *cs);
	
	
	//! Cette methode renvoie un iterator sur la liste des parametres
	std::vector<NsAttribut *>::iterator itBeginParametre();
	
	//! Renvoie true, si l'iterator sur la liste des parametres est  la fin.
	bool atEndParametre(const std::vector<NsAttribut *>::iterator &it);
	
	/** 
	 *Definit le type de cette attribut.
	 *@param t le nouveau type.
	 */
	void setReturnType(const Type &t);
	
	/**
	 *Obtenir le type de cette attribut.
	 *@return le type.
	 */
	Type getReturnType();
	
	//! Obtenir le nom de long de l'objet.
	string getNomComplet();

	//! Definir la methode associ.
	void setMethode(Methode *methode);
		
	//! Obtenir la methode associ.		
	Methode *getMethode();

	//! Obtenir la valeur de _bAbstract.
	bool getAbstract();

	//! Definir la valeur de _bAbstract.
	void setAbstract(bool bAbstract);

	/**
	 * Remplace un noeud syntaxique par un autre.
	 * @param ns l'ancien noeud syntaxique
	 * @param ns2 le remplaant.
	 */
	virtual void remplacerNS(NoeudSyntaxique *ns,NoeudSyntaxique *ns2);
};

#endif

