#ifndef PARPARSER_H
#define PARPARSER_H

#include <iostream>
#include <string>
#include <vector>
#include <stdio.h>

#include <parpilecs.h>

//! Pile des contextes semantiques.
extern PileCS pileCS;

#include<parvisiteurnoeudsyntaxique.h>
#include<parnoeudsyntaxique.h>
#include<parcontextesemantique.h>

//! Permet de grer des chaines de charactre de taille definit.
struct _Chaine {
	char *text;
	int taille;
	};

typedef struct _Chaine Chaine;


class Type;
// Ce fichier n'est pas include si on se trouve dans la compilation du .y

#include<partype.h>
#include<parcstype.h>
#include<parnsattribut.h>
#include<parnsclasse.h>
#include<parnsmethode.h>
#include<parnspaquetage.h>
#include<parerreurparser.h>

//! Indique la ligne du fichier en cours d'analyse.
extern int ligne;

//! Contient la derniere erreur du gner par le parseur C++ ou Java.
extern ErreurParser erreur;
 
// Gestion de la pile des imports pour le java.
#include<parpileimport.h>

//! Pile des imports pour le parser java.
extern std::stack<ImportFile> pileImport;

//! Liste des fichiers import pour le parser java.
extern std::vector<string> listeFichier;

//! Crer un buffer de lecture pour le parser java.
extern void *javaCreateBuffer(FILE *file);

//! Detruit un buffer de lecture pour le parser java.
extern void javaDeleteBuffer(void *buffer);

//! Ouvrir un fichier source pour le parser java.
FILE *javaOuvrirFichierSource(const string &fileName);


//! Lance le parser C++ sur un fichier.
extern bool cppExecuterParser(const string &s);

//! L'analyseur lexicale C++ ignore tout ce qui se trouve jusqu'a la prochaine accolade fermant de mme niveau.
extern void cppIgnorerBrace();

//! L'analyseur lexicale C++ ignore tout les lexemes jusqu'a la prochaine virgule ou point virgule.
extern void cppIgnorerInitVar();

//! L'analyseur lexicale C++ ignore tout les lexemes jusqu'a la prochaine virgule ou parenthese fermante.
extern void cppIgnorerInitParam();

//! Renvoie le prochain lexemes.
extern int cpplex();

//! Lance le parser java sur un fichier.
extern bool javaExecuterParser(const string &s);

//! Lance le parser java sur un repertoire.
extern bool javaExecuterParserRepertoire(const string &s);

//! L'analyseur lexicale java ignore tout ce qui se trouve jusqu'a la prochaine accolade fermant de mme niveau.
extern void javaIgnorerBrace();

//! L'analyseur lexicale java ignore tout les lexemes jusqu'a la prochaine virgule ou point virgule.
extern void javaIgnorerInitVar();

//! L'analyseur lexicale java ignore tout les lexemes jusqu'a la prochaine virgule ou parenthese fermante.
extern void javaIgnorerInitParam();

//! Renvoie le prochain lexemes.
extern int javalex();

#include<parparsercpp.h>
#include<parparserjava.h>


#endif

