%{
#include <parparser.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>

#include <parsersynjava.tab.h>



int javaBrace;
int javaParenthese;
std::vector<string> listeFichier;
FILE *javaOuvrirFichierSource(const string &fileName);
bool javaInsererFichier(const string &s);
bool setNextBufferLex();
std::vector<string> javaCurrentImport;
std::stack<ImportFile> pileImport;

extern int javaparse();

void *javaCreateBuffer(FILE *file) {
	return  yy_create_buffer( file, YY_BUF_SIZE );
	}

void javaDeleteBuffer(void *buffer) {
	yy_delete_buffer((YY_BUFFER_STATE)buffer);
	}

%}

%x ET_COMMENTAIRE
%x ET_COMMENTAIRE_METHODE
%x ET_CHAINE_METHODE

/* 
 * Cette etat est utilise lorsque l'on rencontre l'initialisation d'une variable
 * Dans ce cas, on ignore tout ce qui est compris entre le '{' jusqu' la prochaine
 * de meme niveau '}'.
 */
%x ET_METHODE

/* 
 * Cette etat est utilise lorsque l'on rencontre l'initialisation d'une variable
 * Dans ce cas, on ignore tout ce qui est compris entre le '=' jusqu' la prochaine
 * ',' ou ';'.
 */ 
%x ET_VAR_INIT

/* 
 * Cette etat est utilise lorsque l'on rencontre l'initialisation des parametres d'une methode
 * Dans ce cas, on ignore tout ce qui est compris entre le '=' jusqu' la prochaine ',' ou ';'.
 */
%x ET_PARAM_INIT

/*
 * Ignorer le motif suivant dans les classes.
 * static {
 *	...
 *	}
 * Ces informations ne sont pas transmise  l'analyseur syntaxique.
 */
%x IGNORE_INIT_STATIC_CLASS
%%

"static"\ +"{"	{
		javaBrace = 1;
		javaParenthese = 0;
		BEGIN(IGNORE_INIT_STATIC_CLASS);
		}

"package"	{return KW_package;}

"import"\ +[_a-zA-Z][a-zA-Z0-9_]*(\.[_a-zA-Z][a-zA-Z0-9_]*)*".*"?";"	{
	// On sauvegarde dans la pile des imports la ligne ou l'on se trouve 
	// avant d'aller voir les autres fichiers.
	pileImport.top().setLigneFichierCourant(ligne);

	string s = string(yytext);
	string nomFichier;
	bool bPaquetageRep;
	int i=0;
	while (s[i] != ' ') ++i;
	while (s[i] == ' ') ++i;
	
	nomFichier.assign(s,s.size()-3,s.size());
	bPaquetageRep = (nomFichier == string(".*;"));
	if(bPaquetageRep) {	
		nomFichier.assign(s,i,s.size() - i - 3);
		}
	else	
		{
		nomFichier.assign(s,i,s.size() - i - 1);
		}
	i = 0;
	pileImport.top().ajouterImport(nomFichier);
	while(i < nomFichier.size()) {
		if(nomFichier[i] == '.') {
			nomFichier[i] = '/';
			}
		i++;
		}
	//nomFichier.replace(".","/",0,nomFichier.size());
	if(bPaquetageRep == false)
		{
		nomFichier += string(".java");
		javaInsererFichier(nomFichier);	
		}
	else	
		{
		javaExecuterParserRepertoire(nomFichier);	
		}
	}

"abstract"	{}
"strictfp"	{/* Inconnue comme mot cl */}
"native"	{}

"class"	{return KW_class;}

"public"	{return KW_public;}

"private"	{return KW_private;}

"protected"	{return KW_protected;}

"extends"	{return KW_extends;}

"implements"	{return KW_implements;}

"interface"	{return KW_interface;}

"final"	{}
"static"	{}
"synchronized"	{}
"transient"	{}

"/*"	{
		BEGIN(ET_COMMENTAIRE);
		}
	

<ET_METHODE,INITIAL>"//"[^\n]*\n	{}
<ET_METHODE,INITIAL>"'"."'"	{}
<ET_METHODE>\"	{BEGIN(ET_CHAINE_METHODE);}



"byte"|"boolean"|"char"|"int"|"long"|"short"|"float"|"double"|"void"	{javalval.identifiant.text=javatext;javalval.identifiant.taille=strlen(javatext);return KW_type;}

[_a-zA-Z][a-zA-Z0-9_]* {
	NoeudSyntaxique *ns;
	ns = (NoeudSyntaxique *)pileCS.top()->rechercherNS(string(javatext));
	if(ns == NULL) {
		javalval.identifiant.text = javatext;
		javalval.identifiant.taille = strlen(javatext);
		return IDENTIFIANT_NONDEF;
		}
	else {
		if(ns->getType() == KW_type) {
			javalval.identifiant.text = (char*)(ns->getNom()).c_str();
			javalval.identifiant.taille = (ns->getNom()).size();
			return KW_type;
			}
		else 
			{
			javalval.ns = ns;
			return ns->getType();
			}
	     }
	}


"throws"[^{;]*("{"|";")	{
			// On ignore les throws.
			return yytext[strlen(yytext)-1];
			}
"[]"	{return KW_tableau;}
"*"|"="|":"|"{"|"}"|";"|","|"("|")"|"."	{
	return (char)javatext[0];
	}

" "|"\t"	{}

"\n"	{ligne++;}


<ET_COMMENTAIRE,ET_COMMENTAIRE_METHODE>"\n"	{ligne++;}
<ET_COMMENTAIRE>"*"+"/"	{BEGIN(INITIAL);}
<ET_COMMENTAIRE_METHODE>"*"+"/"	{BEGIN(ET_METHODE);}
<ET_COMMENTAIRE,ET_COMMENTAIRE_METHODE>"*"+[^/]	{}					


<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,IGNORE_INIT_STATIC_CLASS>\"([^\"\\]|\\.)*\"	{}
<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,IGNORE_INIT_STATIC_CLASS>\'.\'	{}

<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,IGNORE_INIT_STATIC_CLASS>"\n"	{++ligne;}
<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,IGNORE_INIT_STATIC_CLASS>"("	{++javaParenthese;}
<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,IGNORE_INIT_STATIC_CLASS>")"	{--javaParenthese;}
<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,IGNORE_INIT_STATIC_CLASS>"{"	{++javaBrace;}
<ET_VAR_INIT,ET_PARAM_INIT>"}"	{--javaBrace;}
<ET_VAR_INIT>";"|","	{
			if((javaParenthese == 0) && (javaBrace == 0)) {
				BEGIN(INITIAL);
				return javatext[0];
				}
			}	
<ET_PARAM_INIT>","|")"	{
			if((javaParenthese == 0) && (javaBrace == 0)) {
				BEGIN(INITIAL);
				return javatext[0];
				}
			}
<ET_METHODE>"}"	{
			--javaBrace;
			if(javaBrace == 0) {
				BEGIN(INITIAL);
				return '}';
				}
			}

<IGNORE_INIT_STATIC_CLASS>"};"|"}"	{
				--javaBrace;
				if(javaBrace == 0) {
					// Fin de IGNORE_INIT_STATIC_CLASS
					BEGIN(INITIAL);
					}
				}

<ET_METHODE>"/*"	{BEGIN(ET_COMMENTAIRE_METHODE);}
					
<ET_METHODE><<EOF>>	{
		cout << "Erreur : Dans les accolades" << endl;
		if(pileImport.size() > 1) {
			pileImport.top().detruireBufferCourant();
			pileCS.pop();
			setNextBufferLex();
			return '}';
			}
		else 	
			{
			yyterminate();	
			}
		}
		
<INITIAL,ET_VAR_INIT,ET_PARAM_INIT><<EOF>>	{
		if(pileImport.size() > 1) {
			pileImport.top().detruireBufferCourant();
			pileCS.pop();
			setNextBufferLex();
			}
		else 	
			{
			yyterminate();	
			}
		}

		
<ET_METHODE,ET_VAR_INIT,ET_PARAM_INIT,ET_COMMENTAIRE,ET_COMMENTAIRE_METHODE,ET_CHAINE_METHODE>.	{/* Ignor */}

%%

bool setNextBufferLex() {
	YY_BUFFER_STATE buffer;
	if(pileImport.size() >= 1) {
		buffer = NULL;
		while((pileImport.size() >= 1) && ((buffer = (YY_BUFFER_STATE)pileImport.top().getBufferCourant()) == NULL)) pileImport.pop();	
		if(pileImport.size() == 0) {
			return false;
			}
		ligne = 1;
		yy_switch_to_buffer(buffer);
		return true;
		}
	return false;
	}


bool javaExecuterParserRepertoire(const string &s) {
	std::vector<string>::iterator it = ParserJava::listeClassPath.begin();
	string sPath;
	DIR *d;	
	d = NULL;
	while((it != ParserJava::listeClassPath.end()) && (d == NULL)) {
		sPath = *it;
		sPath += string("/");
		sPath += s;
		d = opendir(sPath.c_str());	
		if(d == NULL) {
			perror("javaExecuterParserRepertoire:");
			}
		++it;
		}
	
	if(d == NULL) {
		cerr << "javaExecuterParserRepertoire:Le paquetage " << s << " est introuvable" << endl;
		return false;
		}
	
	string nomFichier;
	struct dirent *dEntree;

	ImportFile imp;

	while((dEntree = readdir(d)) != NULL) {
		nomFichier = dEntree->d_name;
		if(nomFichier.size() > 5) { // cad possede au moin l'extension ".java"
			sPath.assign(nomFichier,nomFichier.size()-5,s.size());
			if(sPath == string(".java"))
				imp.ajouterFichierJava(s + string("/") + nomFichier);
			}
		}
	closedir(d);

	pileImport.push(imp);	
	setNextBufferLex();
	return true;
	}


FILE *javaOuvrirFichierSource(const string &s) {
	std::vector<string>::iterator it;
	string sPath;
	FILE *file;
	it = ParserJava::listeClassPath.begin();
	file = NULL;
	while((it != ParserJava::listeClassPath.end()) && (file == NULL)) {	
		sPath = ((string)*it) + "/" + s;
		file = fopen(sPath.c_str(),"r");
		++it;
		}
	
	if(file == NULL) {
		cout << "ERREUR javaOuvrirFichierSource: le fichier " << s << " n'a pas t trouv" << endl;
		}
	return file;
	}

bool javaInsererFichier(const string &s) {
	ImportFile imp;

	imp.ajouterFichierJava(s);
	pileImport.push(imp);	
	setNextBufferLex();
		
	return true;
	}
	
bool javaExecuterParser(const string &s) {
	if(javaInsererFichier(s) == false) {
		return false;
		}
	ImportFile imp;
	imp.ajouterFichierJava("java/lang/Object.java");
	imp.ajouterFichierJava("java/lang/Class.java");
	pileImport.push(imp);	

	ligne = 1;
	javaparse();
	return true;
	}

void javaIgnorerBrace() {
	javaBrace = 1;
	BEGIN(ET_METHODE);	
	}
	
void javaIgnorerInitVar() {
	javaParenthese=0;
	javaBrace = 0;
	BEGIN(ET_VAR_INIT);
	}
	
void javaIgnorerInitParam() {
	javaParenthese=0;
	javaBrace=0;
	BEGIN(ET_PARAM_INIT);
	}	

int yywrap ( void ) {
	return 1;
	}
